@@ -119,3 +119,17 @@ getdtablesize()
 }
 
 #endif	/* USE_NOFILE */
+
+/* Debugging stuff.. */
+int countfds()
+{
+  int fds = getdtablesize();
+  int i;
+  int cnt = 0;
+
+  for (i=0; i<fds; ++i)
+    if (fcntl(i,F_GETFL) >= 0)
+      ++cnt;
+
+  return cnt;
+}
