@@ -211,11 +211,11 @@ register int timeout;
  fd_set wfds;
  register int r;
  register int saveerrno;
- void* (*old_sig)();
+ void (*old_sig)__((int));
  char *retval;
  
 
- old_sig = (void*)signal(SIGPIPE, SIG_IGN);
+ old_sig = signal(SIGPIPE, SIG_IGN);
  
  s = auth_tcpsock(inlocal,inremote);
  if (s == -1)
@@ -260,9 +260,9 @@ register unsigned short remote;
  unsigned short rremote;
  register int fl;
  fd_set wfds;
- void* (*old_sig)();
+ void (*old_sig)__((int));
  
- old_sig = (void*)signal(SIGPIPE, SIG_IGN);
+ old_sig = signal(SIGPIPE, SIG_IGN);
  
  FD_ZERO(&wfds);
  FD_SET(s,&wfds);
