@@ -1039,7 +1039,7 @@ sh_wait(argc, argv)
 		return 0;
 #if	defined(WNOHANG) && !defined(WLOBYTE)
 	if (status.w_termsig) {
-		fprintf(stderr, "%s", strsignam(status.w_termsig));
+		fprintf(stderr, "%s", strsignal(status.w_termsig));
 		if (status.w_coredump)
 			fprintf(stderr, CORE_DUMPED);
 		fprintf(stderr, "\n");
