@@ -557,9 +557,11 @@ rd_doit(filename, dirs)
 	static char *buf;
 #endif
 	char *av[3];
+	char *p;
 	int len;
 	char pathbuf[512];
 	char *sh_memlevel = getlevel(MEM_SHCMD);
+	int thatpid;
 
 	*pathbuf = 0;
 	if (*dirs) {	/* If it is in alternate dir, move to primary one,
@@ -569,54 +571,57 @@ rd_doit(filename, dirs)
 	}
 	strcat(pathbuf,filename);
 
-	if (router_id) {
-	  char *p;
-
-	  len = strlen(filename);
-	  if ((p=strchr(filename, '-')) != NULL) {
-	    /* message file is "file-#" */
-	    if (router_id != atoi(p+1)) {
-	      /*
-	       * an already locked message file,
-	       * belonging to another process
-	       */
-	      if (*dirs) {
-		fprintf(stderr,
-			"** BUG **: %s%s not in primary router directory!\n",
-			dirs,filename);
-	      }
-	      return 0;
-	      /*
-	       * This should not happen anywhere but at
-	       * primary router directory.  If  xxxx-nn
-	       * format file exists anywhere else, it is
-	       * a bug time!
-	       */
-	    }
+	len = strlen(filename);
+	thatpid = 0;
+	if ((p=strchr(filename, '-')) != NULL) {
+	  /* message file is "inode-pid" */
+	  thatpid = atoi(p+1);
+	  if (thatpid < 10) {	/* old-style locking.. */
+	    thatpid = 0;
 	  }
-	  else {
+	  /* Probe it!
+	     Does the process exist ? */
+	  if (thatpid && kill(thatpid,0)==0) {
+	    /*
+	     * an already locked message file,
+	     * belonging to another process
+	     */
+	    if (*dirs) {
+	      fprintf(stderr,
+		      "** BUG **: %s%s not in primary router directory!\n",
+		      dirs,filename);
+	    }
+	    return 0;
+	    /*
+	     * This should not happen anywhere but at
+	     * primary router directory.  If  xxxx-nn
+	     * format file exists anywhere else, it is
+	     * a bug time!
+	     */
+	  }
+	}
+	if (p == NULL || thatpid != getpid()) {
+	  /* Not already in format of 'inode-pid' */
+	  /* If the pid did exist, we do not touch on that file,
+	     on the other hand, we need to rename the file now.. */
 #ifdef	USE_ALLOCA
-	    buf = alloca(len+16);
+	  buf = alloca(len+16);
 #else
-	    if (blen == 0) {
-	      blen = len+16;
-	      buf = (char *)malloc(len+16);
-	    }
-	    while (len + 12 > blen) {
-	      blen = 2 * blen;
-	      buf = (char *)realloc(buf, blen);
-	    }
-#endif
-	    sprintf(buf, "%s-%d", filename, router_id);
-	    if (eqrename(pathbuf, buf, 1) < 0)
-	      return 0;		/* something is wrong, erename() complains */
-	    filename = buf;
-	    /* message file is now "file-#" and belongs to this process */
+	  if (blen == 0) {
+	    blen = len+16;
+	    buf = (char *)malloc(len+16);
 	  }
-	} else
-	  if (*dirs) /* No multirouter, but still multi-priority.. */
-	    if (eqrename(pathbuf,filename, 1) < 0)
-	      return 0; /* Umm.. Don't process if you can't lock it.. */
+	  while (len + 12 > blen) {
+	    blen = 2 * blen;
+	    buf = (char *)realloc(buf, blen);
+	  }
+#endif
+	  sprintf(buf, "%s-%d", filename, getpid());
+	  if (eqrename(pathbuf, buf, 1) < 0)
+	    return 0;		/* something is wrong, erename() complains */
+	  filename = buf;
+	  /* message file is now "file-#" and belongs to this process */
+	}
 
 #ifdef	MALLOC_TRACE
 mal_contents(stdout);
@@ -1074,14 +1079,14 @@ run_praliases(argc, argv)
 		status = PERR_BADOPEN;
 	} else {
 		setvbuf(e->e_fp, buf, _IOFBF, sizeof buf);
-		osiop = siofds[fileno(e->e_fp)];
-		siofds[fileno(e->e_fp)] = NULL;
+		osiop = siofds[FILENO(e->e_fp)];
+		siofds[FILENO(e->e_fp)] = NULL;
 		e->e_file = argv[optind];
 		status = makeLetter(e, 1);	/* Parse the aliases database
 						   as if all entries were of
 						   same syntax as "To:" et.al.
 						   on the normal email header*/
-		siofds[fileno(e->e_fp)] = osiop;
+		siofds[FILENO(e->e_fp)] = osiop;
 		fclose(e->e_fp);
 		e->e_fp = NULL;
 	}
@@ -1134,8 +1139,8 @@ run_praliases(argc, argv)
 			return 5;
 		}
 		setvbuf(indexfp, ibuf, _IOFBF, sizeof ibuf);
-		osiop = siofds[fileno(indexfp)];
-		siofds[fileno(indexfp)] = NULL;
+		osiop = siofds[FILENO(indexfp)];
+		siofds[FILENO(indexfp)] = NULL;
 	} else
 		indexfp = NULL;
 	maxsize = size = 0;
@@ -1175,7 +1180,7 @@ run_praliases(argc, argv)
 	if (fflush(stdout) == EOF)
 		++errflg;
 	if (indexfp != NULL) {
-		siofds[fileno(indexfp)] = osiop;
+		siofds[FILENO(indexfp)] = osiop;
 		if (fclose(indexfp) == EOF)
 			++errflg;
 	}
@@ -1416,8 +1421,8 @@ run_listexpand(avl, il)
 	    if (erroraddress != NULL) {
 	      if (!iserrmessage()
 		  && (mfp = mail_open(MSG_RFC822)) != NULL) {
-		osiop = siofds[fileno(mfp)];
-		siofds[fileno(mfp)] = NULL;
+		osiop = siofds[FILENO(mfp)];
+		siofds[FILENO(mfp)] = NULL;
 		fprintf(mfp, "channel error\n");
 		fprintf(mfp, "to %s\n", erroraddress);
 		fprintf(mfp, "From: Error Channel <MAILER-DAEMON>\n");
@@ -1426,7 +1431,7 @@ run_listexpand(avl, il)
 		fprintf(mfp, "Subject: Error in %s\n", comment);
 		fprintf(mfp, "Precedence: junk\n\n");
 		hdr_errprint(e, &hs, mfp, comment);
-		siofds[fileno(mfp)] = osiop;
+		siofds[FILENO(mfp)] = osiop;
 		mail_close(mfp);
 	      }
 	      printf("%s\n", erroraddress);
@@ -1737,8 +1742,8 @@ run_listaddrs(argc, argv)
 	    if (erroraddress != NULL) {
 	      if (!iserrmessage()
 		  && (mfp = mail_open(MSG_RFC822)) != NULL) {
-		osiop = siofds[fileno(mfp)];
-		siofds[fileno(mfp)] = NULL;
+		osiop = siofds[FILENO(mfp)];
+		siofds[FILENO(mfp)] = NULL;
 		fprintf(mfp, "channel error\n");
 		fprintf(mfp, "to %s\n", erroraddress);
 		fprintf(mfp, "From: Error Channel <MAILER-DAEMON>\n");
@@ -1747,7 +1752,7 @@ run_listaddrs(argc, argv)
 		fprintf(mfp, "Subject: Error in %s\n", comment);
 		fprintf(mfp, "Precedence: junk\n\n");
 		hdr_errprint(e, &hs, mfp, comment);
-		siofds[fileno(mfp)] = osiop;
+		siofds[FILENO(mfp)] = osiop;
 		mail_close(mfp);
 	      }
 	      printf("%s\n", erroraddress);
@@ -1862,7 +1867,7 @@ run_filepriv(argc, argv)
 				"%s: cannot fopen(\"%s\")!\n", argv0, file);
 			return 2;
 		}
-		if (fstat(fileno(fp), &stbuf) < 0) {
+		if (fstat(FILENO(fp), &stbuf) < 0) {
 			fprintf(stderr, "%s: cannot fstat(\"%s\")!\n",
 				argv0, file);
 			fclose(fp);
