@@ -225,11 +225,22 @@ owner_ndbm(sip, outfp)
 
 	if ((db = open_ndbm(sip, O_RDONLY, "owner_ndbm")) == NULL)
 		return;
+	/* There are more timing hazards, when the internal fd is not
+	   available for probing.. */
+#if 1
+	/* No lstat()! */
+	if (stat(sip->file, &stbuf) < 0) {
+		fprintf(stderr, "owner_ndbm: cannot stat(\"%s\")!\n",
+				sip->file);
+		return;
+	}
+#else
 	if (fstat(dbm_pagfno(db), &stbuf) < 0) {
 		fprintf(stderr, "owner_ndbm: cannot fstat(\"%s\")!\n",
 				sip->file);
 		return;
 	}
+#endif
 	fprintf(outfp, "%d\n", stbuf.st_uid);
 	fflush(outfp);
 }
@@ -247,12 +258,19 @@ modp_ndbm(sip)
 	if (sip->file == NULL
 	    || (db = open_ndbm(sip, O_RDONLY, "owner_ndbm")) == NULL)
 		return 0;
+#if 1
+	if (stat(sip->file, &stbuf) < 0) {
+		fprintf(stderr, "modp_ndbm: cannot stat(\"%s\")!\n",
+				sip->file);
+		return 0;
+	}
+#else
 	if (fstat(dbm_pagfno(db), &stbuf) < 0) {
 		fprintf(stderr, "modp_ndbm: cannot fstat(\"%s\")!\n",
 				sip->file);
 		return 0;
 	}
-
+#endif
 	symid = symbol((u_char *)sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
