@@ -106,9 +106,9 @@ run_rfc822(argc, argv)
 		status = PERR_BADOPEN;
 	} else {
 /* XX: DEBUG STUFF! */
-if (fileno(e->e_fp) < 4) {
+if (FILENO(e->e_fp) < 4) {
   fprintf(stderr,"RFC822: While opening mail-file '%s', got fd=%d  AARGH! (abort!)",
-	  file, fileno(e->e_fp));
+	  file, FILENO(e->e_fp));
   fflush(stderr);
   abort();
 }
@@ -192,7 +192,7 @@ makeLetter(e, octothorp)
 	e->e_hdrOffset = 0;
 	e->e_msgOffset = 0;
 	e->e_nowtime = now = time(NULL);
-	if (efstat(fileno(e->e_fp), &(e->e_statbuf)) < 0) {
+	if (efstat(FILENO(e->e_fp), &(e->e_statbuf)) < 0) {
 #ifdef	USE_STATBLKSIZE
 		e->e_statbuf.st_blksize = 0;
 #endif	/* !USE_STATBLKSIZE */
@@ -207,6 +207,15 @@ makeLetter(e, octothorp)
 
 	inheader = 0;
 	while ((n = getline(e->e_fp)) > !octothorp) {
+		/* We do kludgy processing things in case the input
+		   does have a CRLF at the end of the line.. */
+		if (n > 1 &&
+		    linebuf[n-2] == '\r' &&
+		    linebuf[n-1] == '\n') {
+			--n;
+			linebuf[n-1] = '\n';
+		}
+		/* Ok, now we can proceed with the original agenda.. */
 		i = hdr_status(linebuf, linebuf, n);
 		if (i > 0) {		/* a real message header */
 			if (octothorp && linebuf[0] == '#')
@@ -645,8 +654,8 @@ defer(e)
 	char *path;
 
 	if (e->e_fp != NULL && files_gid >= 0) {
-		fchown(fileno(e->e_fp), -1, files_gid);
-		fchmod(fileno(e->e_fp),
+		fchown(FILENO(e->e_fp), -1, files_gid);
+		fchmod(FILENO(e->e_fp),
 			(0440|e->e_statbuf.st_mode) & 0660);
 	}
 #ifdef	USE_ALLOCA
@@ -1119,6 +1128,7 @@ sequencer(e, file)
 	/* put pertinent message information in the zsh environment */
 	setenvinfo(e);
 	if (perr) {
+		/* XX: report this ??? */
 		eunlink(file);
 		return perr;
 	}
@@ -1410,6 +1420,10 @@ sequencer(e, file)
 	if (h == NULL)
 	  FindHeader("to",!e->e_resent);
 	if (h == NULL) {
+#if 1
+		dprintf("No 'To:' -header, creating our own\n");
+		InsertHeader(h, mkToHeader(e,"unlisted-recipients:; (no To-header on input)"));
+#else
 		dprintf("Insert the To: header lines\n");
 		for (h = e->e_headers; h != NULL; h = h->h_next)
 			if (CISTREQ(h->h_pname, "subject"))
@@ -1434,6 +1448,7 @@ sequencer(e, file)
 				/* so we add them in reverse order... */
 			}
 		}
+#endif
 	}
 #ifdef notdef
 	rewrite all addresses in the message according to the
@@ -1594,6 +1609,12 @@ sequencer(e, file)
 		}
 	}
 
+	isSenderAddr = isRecpntAddr = 0;
+	dprintf("Make sure Date, From, To, are in the header\n");
+	FindHeader("date",e->e_resent);
+	if (h == NULL)
+	  InsertHeader(h, mkDate(e->e_resent, e->e_statbuf.st_mtime));
+
 	dprintf("Rewrite message headers\n");
 	for (rwp = rwhead; rwp != NULL; rwp = rwp->next) {
 		hp = &rwp->urw.h;
@@ -1610,23 +1631,17 @@ sequencer(e, file)
 		*hp = NULL;
 	}
  
-	isSenderAddr = isRecpntAddr = 0;
-	dprintf("Make sure Date, From, To, are in the header\n");
-	FindHeader("date",e->e_resent);
-	if (h == NULL)
-	  InsertHeader(h, mkDate(e->e_resent, e->e_statbuf.st_mtime));
-
 	if (rwhead == NULL)
 	  return PERR_NORECIPIENTS;
 
 	dprintf("Emit specification to the transport system\n");
 #ifdef	USE_ALLOCA
-	ofpname = (char *)alloca((u_int)(strlen(file)+2));
+	ofpname = (char *)alloca((u_int)(strlen(file)+9));
 #else
-	ofpname = (char *)emalloc((u_int)(strlen(file)+2));
+	ofpname = (char *)emalloc((u_int)(strlen(file)+9));
 #endif
-	ofpname[0] = '.';
-	strcpy(&ofpname[1], file);
+	/* Guaranteed unique within this machine */
+	sprintf(ofpname,".%s.%d",file,getpid());
 	if ((ofp = fopen(ofpname, "w+")) == NULL) {
 #ifndef USE_ALLOCA
 		free(ofpname);
@@ -1636,8 +1651,8 @@ sequencer(e, file)
 	}
 	setvbuf(ofp, vbuf, _IOFBF, sizeof vbuf);
 	if (files_gid >= 0) {
-		fchown(fileno(ofp), e->e_statbuf.st_uid, files_gid);
-		fchmod(fileno(ofp), 0460);
+		fchown(FILENO(ofp), e->e_statbuf.st_uid, files_gid);
+		fchmod(FILENO(ofp), 0460);
 	}
 	fprintf(ofp, "%c%c%s\n",
 		_CF_MESSAGEID, _CFTAG_NORMAL, file);
@@ -1847,8 +1862,8 @@ sequencer(e, file)
 	}
 	
 	if (e->e_fp != NULL && files_gid >= 0) {
-		fchown(fileno(e->e_fp), -1, files_gid);
-		fchmod(fileno(e->e_fp),
+		fchown(FILENO(e->e_fp), -1, files_gid);
+		fchmod(FILENO(e->e_fp),
 		       (0440|e->e_statbuf.st_mode) & 0660);
 	}
 	/* all is nirvana -- link the input file to somewhere safe */
