@@ -319,6 +319,33 @@ mkMessageId(e, unixtime)
 	return h;
 }
 
+struct header *
+mkToHeader(e, buf)
+	struct envelope *e;
+	const char *buf;
+{
+	register struct header *h;
+	static struct headerinfo *toDesc;
+	
+	if (toDesc == NULL) {
+		for (toDesc = &mandatory_hdrs[0];
+		     toDesc->hdr_name != NULL; ++toDesc) {
+			if (toDesc->semantics == AddressList)
+				break;
+		}
+	}
+	h = (struct header *)tmalloc(sizeof (struct header));
+	h->h_pname = e->e_resent ? "Resent-To" : "To";
+	h->h_descriptor = toDesc;
+	h->h_stamp = newHeader;
+	h->h_next = 0;
+	h->h_lines = makeToken(buf, strlen((char *)buf));
+	h->h_lines->t_type = Line;
+	h->h_contents = hdr_scanparse(e, h, 0);
+	/* OPTIMIZE: save this in a hashtable somewhere */
+	return h;
+}
+
 /*
  * Construct an RFC822 date header.
  */
@@ -345,7 +372,7 @@ mkDate(isresent, unixtime)
 	h->h_stamp = newHeader;
 	h->h_next = 0;
 	h->h_contents.d = unixtime;
-	cp = rfc822date(&(h->h_contents.d));
+	cp = (u_char*)rfc822date(&(h->h_contents.d));
 	s = cp + strlen((char*)cp) -1;
 	if (*cp != 0 && *s == '\n') *s = 0;
 	h->h_lines = makeToken(cp, strlen((char *)cp));
@@ -1044,10 +1071,10 @@ hdr_warning(h)
 
 	wh->h_lines = makeToken((u_char *)cp, strlen(cp));
 
-	sb_external(fileno(stdout));
+	sb_external(FILENO(stdout));
 	printf("\t%s:", h->h_pname);
 	errprint(stdout, hwap->a_tokens, strlen(h->h_pname)+8);
-	cp = sb_retrieve(fileno(stdout));
+	cp = sb_retrieve(FILENO(stdout));
 	len = strlen(cp);
 	if (*(cp + len - 1) == '\n')
 		--len;
@@ -1090,11 +1117,11 @@ hdr_errprint(e, h, fp, msg)
 		FILE *ealfp;
 
 		if ((ealfp = fopen(erraddrlog, "a")) != NULL) {
-			struct siobuf *osiop = siofds[fileno(ealfp)];
+			struct siobuf *osiop = siofds[FILENO(ealfp)];
 			char vbuf[BUFSIZ];
 
 			setvbuf(ealfp, vbuf, _IOFBF, sizeof vbuf);
-			siofds[fileno(ealfp)] = NULL;
+			siofds[FILENO(ealfp)] = NULL;
 			if (e != NULL)
 				fprintf(ealfp, "%s: %s\n",
 				       e->e_file, rfc822date(&(e->e_nowtime)));
@@ -1109,7 +1136,7 @@ hdr_errprint(e, h, fp, msg)
 				}
 			putc('\n', ealfp);
 			fclose(ealfp);
-			siofds[fileno(ealfp)] = osiop;
+			siofds[FILENO(ealfp)] = osiop;
 		}
 	}
 }
