@@ -24,7 +24,9 @@
 #include "prototypes.h"
 
 extern char *strchr();
+extern char *strrchr();
 extern char *strnsave();
+extern int errno;
 
 /*
  * There has to be some way of collecting the error messages produced during
@@ -41,6 +43,7 @@ struct not {
 	char *not;
 	char *dsn;
 	char *rcpt;
+	char *rcpntp;
 };
 
 static void	/* There is notaryreport() on transporters also.. */
@@ -49,6 +52,11 @@ scnotaryreport(errfp,notary)
 	struct not *notary;
 {
 	char *rcpt, *action, *status, *diagnostic, *wtt;
+	char const *typetag;
+	char *channel;
+
+	static char const *type_rfc   = "RFC822";
+	static char const *type_local = "X-LOCAL";
 
 	/* NOTARY: addres / action / status / diagnostic / wtt */
 
@@ -65,26 +73,56 @@ scnotaryreport(errfp,notary)
 	status   = strchr(action,'\001');
 	*status++ = 0;
 	diagnostic = strchr(status,'\001');
-	*diagnostic++ = 0;
-	wtt      = strchr(diagnostic,'\001');
-	*wtt++    = 0;
+	if (diagnostic == NULL) {
+	  diagnostic = status;
+	  status = NULL;
+	  wtt = NULL;
+	} else {
+	  *diagnostic++ = 0;
+	  wtt      = strchr(diagnostic,'\001');
+	  *wtt++    = 0;
+	}
 
-	if (strchr(rcpt,'@') != NULL)
-	  fprintf(errfp, "Final-Recipient: RFC822; %s\n", rcpt);
-	else
-	  fprintf(errfp, "Final-Recipient: X-LOCAL; %s\n", rcpt);
+	if (*rcpt == '\003' /* XX: MAGIC! */) {
+	  /* Tough, internal expiry didn't know our target address!
+	     Lets scan it from  rcpntp .. We can mush the string pointed
+	     by the rcpntp.. */
+	  char *s = notary->rcpntp;
+	  char *p;
+	  while (*s != ' ' && *s != '\t' && *s != 0) ++s;
+	  while (*s && (*s == ' ' || *s == '\t')) ++s;
+	  /* XX: If the 'host' part has spaces in it ?? */
+	  while (*s != ' ' && *s != '\t' && *s != 0) ++s;
+	  while (*s && (*s == ' ' || *s == '\t')) ++s;
+	  p = strrchr(s,' ');
+	  if (!p) p = strrchr(s,'\t');
+	  if (p) *p = 0;
+	  /* Ok, we have it chopped ..  Quite a butcher job, but.. */
+	  rcpt = s;
+	}
+
+	if (strchr(rcpt,'@') != NULL) {
+	  typetag = type_rfc;
+	  if (strncmp(rcpt,"ns:",3)==0) /* 'hold'-channel stuff */
+	    typetag = type_local;
+	} else
+	  typetag = type_local;
+	fprintf(errfp, "Final-Recipient: %s; %s\n", typetag, rcpt);
 	fprintf(errfp, "Action: %s\n", action);
-	fprintf(errfp, "Status: %s\n", status);
+	if (status)
+	  fprintf(errfp, "Status: %s\n", status);
 	fprintf(errfp, "Diagnostic-Code: %s\n", diagnostic);
 	if (notary->dsn)
 	  fprintf(errfp, "X-DSN: %s\n", notary->dsn);
-	if (wtt[0] != 0)
+	if (wtt && wtt[0] != 0)
 	  fprintf(errfp, "Remote-MTA: %s\n", wtt);
 	fprintf(errfp, "\n");
 	action[-1] = '\001';
-	status[-1] = '\001';
+	if (status)
+	  status[-1] = '\001';
 	diagnostic[-1] = '\001';
-	wtt[-1]    = '\001';
+	if (wtt)
+	  wtt[-1]    = '\001';
 }
 
 
@@ -138,6 +176,7 @@ reporterrs(cfpi)
 	int notaryspc = 0;
 	char *dnsrecipient = NULL;
 	struct ctlfile *cfp;
+	char *rcpntpointer;
 
 	if (cfpi->haderror == 0)
 		return;
@@ -162,14 +201,14 @@ reporterrs(cfpi)
 	lp = &cfp->offset[0];
 	if ((errfp = mail_open(MSG_RFC822)) == NULL) {
 	  fprintf(stderr,
-		  "%s: cannot open mail file to return diagnostics!\n",
-		  progname);
+		  "%s: cannot open output mail file to return diagnostics! errno=%d, errstr='%s'\n",
+		  progname, errno, strerror(errno));
 	  close(cfp->fd);
-	  free(cfp->contents);
-	  free((char *)cfp);
+	  free_cfp_memory(cfp);
 	  return;
 	}
-	cfp->mid = midbuf;
+	if (cfp->mid) free(cfp->mid);
+	cfp->mid = midbuf; /* This is the original one! */
 	wroteheader = 0;
 	for (i = 0; i < cfp->nlines; ++i, ++lp) {
 	  cp = cfp->contents + *lp;
@@ -247,6 +286,7 @@ reporterrs(cfpi)
 	    notary = strnsave(notary, cp-notary); /* Make a copy of it */
 	    ++cp;
 	  }
+	  rcpntpointer = cfp->contents + byteidx + 2 + _CFTAG_RCPTPIDSIZE;
 	  if (notary != NULL && *notary != 0) {
 	    if (!notaries) {
 	      notaryspc = 8;
@@ -257,9 +297,10 @@ reporterrs(cfpi)
 	      notaries = (void*)erealloc((void*)notaries,
 					 sizeof(struct not)*(notaryspc+1));
 	    }
-	    notaries[notarycnt].not = notary;
-	    notaries[notarycnt].dsn = NULL;
-	    notaries[notarycnt].rcpt = dnsrecipient;
+	    notaries[notarycnt].not    = notary;
+	    notaries[notarycnt].dsn    = NULL;
+	    notaries[notarycnt].rcpt   = dnsrecipient;
+	    notaries[notarycnt].rcpntp = rcpntpointer;
 	    if (drptidx > 0) {
 	      notaries[notarycnt].dsn = cfp->contents + drptidx;
 	    }
@@ -270,7 +311,7 @@ reporterrs(cfpi)
 	  /* Scan to the start of the message text */
 	  while (isascii(*cp) && isspace(*cp))
 	    ++cp;
-	  fprintf(errfp, "<%s>: ", cfp->contents + byteidx + 2 + _CFTAG_RCPTPIDSIZE);
+	  fprintf(errfp, "<%s>: ", rcpntpointer);
 	  if (strchr(cp, '\r')) {
 	    fprintf(errfp, "...\\\n\t");
 	    for (s = cp; *s != '\0'; ++s) {
@@ -306,6 +347,8 @@ reporterrs(cfpi)
 	/* Now scan 'em all again for IETF-NOTARY */
 	for (i = 0; i < notarycnt; ++i) {
 	  scnotaryreport(errfp, &notaries[i]);
+	  if (notaries[i].not != NULL)
+	    free(notaries[i].not);
 	}
 	free((void*)notaries);
 
@@ -350,8 +393,8 @@ reporterrs(cfpi)
 
 	mail_close(errfp);	/* XX: check for error */
 	close(cfp->fd);
-	free(cfp->contents);
-	free((char *)cfp);
+	cfp->mid = NULL; /* we don't want to loose the original one! */
+	free_cfp_memory(cfp);
 }
 
 char *
