@@ -11,6 +11,18 @@
 # endif
 #endif
 
+#if 0
+/* *** DEBUGGING STUFF! *** */
+
+static void *emalloc(size) size_t size; { void *ptr = malloc(size); fprintf(stderr,"mal: siz=%d ptr=0x%p\n",size,ptr); return ptr; }
+static void *erealloc(ptr,size) void *ptr; size_t size; {void *newptr = realloc(ptr,size);fprintf(stderr,"mal: realloc(0x%p,%d)->0x%p\n",ptr,size,newptr);return newptr; }
+static char *strsave(s) const char *s; { char *s2 = emalloc(strlen(s)+1); strcpy(s2,s); return s2;}
+static char *strnsave(s,len) const char *s; const size_t len; { char *s2 = emalloc(len+1); strncpy(s2,s,len); s2[len] = 0; return s2;}
+static void _free(ptr,file,line) void *ptr; char*file;int line; { fprintf(stderr,"%s:%d:mal: free(0x%p)\n",file,line,ptr); free(ptr); }
+#define free(ptr) _free(ptr,__FILE__,__LINE__)
+
+#endif
+
 /* agenda.c */
 extern time_t qipcretry;
 extern int doagenda __((void));
@@ -67,6 +79,7 @@ extern int  global_maxkids;
 extern int  verbose;
 extern int  querysocket;
 extern struct ctlfile *slurp __((int fd, long ino));
+extern void free_cfp_memory __((struct ctlfile *cfp));
 extern int  vtxredo __((struct spblk *spl));
 extern char *timestring __((void));
 extern time_t now;
@@ -74,10 +87,10 @@ extern FILE *statuslog;
 
 /* threads.c */
 extern struct thread *thread_head, *thread_tail;
-extern struct threadgroup *create_threadgroup __((struct config_entry *cep, struct web *wc, struct web *wh, int withhost));
+extern struct threadgroup *create_threadgroup __((struct config_entry *cep, struct web *wc, struct web *wh, int withhost, void (*ce_fillin)__((struct threadgroup *, struct config_entry *)) ));
 extern void  delete_threadgroup __((struct threadgroup *thgp));
-extern struct thread *create_thread __((struct threadgroup *thgrp, struct vertex *vtx));
-extern void  thread_linkin __((struct vertex *cp, struct config_entry *cep, int cfgid));
+extern void  thread_linkin __((struct vertex *cp, struct config_entry *cep, int cfgid, void (*ce_fillin)__((struct threadgroup *, struct config_entry *)) ));
+extern char  *strlower __((char *s));
 extern struct web *web_findcreate __((int flag, u_char *s));
 extern void        unweb __((int flag, struct web *wp));
 extern int   thread_start __((struct thread *thr));
@@ -111,6 +124,5 @@ extern void deletemsg __((char *, struct ctlfile *));
 extern char *saytime __((long, char *, int));
 extern void expire __((struct vertex *, int));
 
-
 /* Transport library */
 extern int lockaddr __((int, long, char, char));
