@@ -14,6 +14,12 @@
 #include <sys/file.h>
 #include "prototypes.h"
 
+
+/* define NO_VERBOSE_MAILQ, if you don't want to output potentially HUGE
+   mailq report -- necessary at some sites, while definitely undesirable
+   at others..  We need to change this mechanism.. */
+#define NO_VERBOSE_MAILQ
+
 extern int errno;
 extern time_t time();
 
@@ -74,12 +80,18 @@ qprint(fd)
 
 	fprintf(qpfp, "version zmailer 1.0\nVertices:\n");
 	time(&qpnow);
-	sp_scan(qpctlfile, (struct spblk *)NULL, spt_mesh[L_CTLFILE]);
+#ifndef NO_VERBOSE_MAILQ
+	sp_scan(qpctlfile, (struct spblk *)NULL, spt_mesh[L_CTLFILE]); 
+#endif
 	if (qpch != '\0') {
 		fprintf(qpfp, "Channels:\n");
+#ifndef NO_VERBOSE_MAILQ
 		sp_scan(qpchannel, (struct spblk *)NULL, spt_mesh[L_CHANNEL]);
+#endif
 		fprintf(qpfp, "Hosts:\n");
+#ifndef NO_VERBOSE_MAILQ
 		sp_scan(qphost, (struct spblk *)NULL, spt_mesh[L_HOST]);
+#endif
 	}
 	fprintf(qpfp, "End:\n");
 
@@ -131,10 +143,10 @@ static int qpctlfile(spl)
 	    saytime((u_long)(vp->wakeup - qpnow), buf, 1);
 	    fprintf(qpfp, "(retry in %s", buf);
 	    ++i;
-	  } else if (vp->ce.pending) {
+	  } else if (vp->ce_pending) {
 	    fprintf(qpfp, "(waiting for %sslot",
-		    vp->ce.pending == SIZE_L ? "" :
-		    vp->ce.pending == L_CHANNEL ? "channel " :
+		    vp->ce_pending == SIZE_L ? "" :
+		    vp->ce_pending == L_CHANNEL ? "channel " :
 		    "thread ");
 	    ++i;
 	  } else {
@@ -161,10 +173,10 @@ static int qpctlfile(spl)
 	    }
 	    ++i;
 	  }
-	  if (vp->ce.expiry > 0) {
+	  if (vp->ce_expiry > 0) {
 	    /* [mea] Want to know when it expires.. */
 	    buf[0] = '\0';
-	    saytime((long)(vp->ce.expiry - qpnow), buf, 1);
+	    saytime((long)(vp->ce_expiry - qpnow), buf, 1);
 	    fprintf(qpfp,"%sexpires in %s, tries=%d",
 		    i ? ", " : "(", buf, vp->attempts);
 	    }
