@@ -68,19 +68,22 @@ int	querysocket = -1;	/* fd of TCP socket to listen for queries */
 int	D_alloc = 0;
 int	hungry_childs = 0;
 int	global_wrkcnt = 0;
+int	syncstart = 0;		/* while set, the thread subsystem shall start
+				   no childs! */
 static int vtxprep_skip      = 0;
 static int vtxprep_skip_any  = 0;
 static int vtxprep_skip_lock = 0;
 static time_t next_dirscan     = 0;
 static time_t next_idlecleanup = 0;
 static struct sptree *dirscan_mesh = NULL;
+static int newents_limit = 200;
 
 #include "memtypes.h"
 extern memtypes stickymem;
 
 static struct ctlfile *schedule __((int fd, char *file, struct config_entry *ce, long ino));
 static struct ctlfile *vtxprep __((struct ctlfile *));
-static int  vtxfill __((struct vertex *, struct config_entry *));
+static int  vtxmatch __((struct vertex *, struct config_entry *));
 static void link_in __((int flag, struct vertex *vp, u_char *s));
 static int  lockverify __((struct ctlfile *, char *));
 static int  globmatch   __((char *, char*));
@@ -113,6 +116,8 @@ extern void die __((int rc, char *reason));
 extern int  killprevious __((int signal, char *pidfile));
 extern void prversion __((char *name));
 
+extern int thread_count_recipients();
+
 struct dirstatname {
 	struct stat st;
 	long ino;
@@ -151,8 +156,7 @@ struct ctlfile *cfp;
 	    unvertex(vp,1,1); /* Don't unlink! Just free()! */
 	  }
 	} else {
-	  free(cfp->contents);
-	  free((char *)cfp);
+	  free_cfp_memory(cfp);
 	}
 }
 
@@ -164,6 +168,24 @@ struct spblk *spl;
 	return 0;
 }
 
+
+/*
+ *  free_cfp_memory(cfp) -- release all of the memory associated with the
+ *  control file -- some of it is conditionally allocated, or perhaps
+ *  previously released..  (cfp->contents, for example)
+ */
+void free_cfp_memory(cfp)
+struct ctlfile *cfp;
+{
+	if (cfp->contents)	free(cfp->contents);
+	if (cfp->mid)		free(cfp->mid);
+	if (cfp->erroraddr)	free(cfp->erroraddr);
+	if (cfp->logident)	free(cfp->logident);
+	if (cfp->envid)		free(cfp->envid);
+	free((char *)cfp);
+}
+
+
 int
 main(argc, argv)
 	int argc;
@@ -202,12 +224,18 @@ main(argc, argv)
 	postoffice = rendezvous = log = config = NULL;
 	daemonflg = 1;
 	dlyverbose = 0;
+	syncstart = 0;
 	verbose = errflg = version = 0;
-	while ((c = getopt(argc, argv, "divf:Fl:L:N:P:Q:VW")) != EOF)
+	while ((c = getopt(argc, argv, "divE:f:Fl:L:N:P:Q:SVW")) != EOF)
 		switch (c) {
 		case 'f':	/* override default config file */
 			config = optarg;
 			break;
+		case 'E':
+			newents_limit = atoi(optarg);
+			if (newents_limit < 10)
+			  newents_limit = 10;
+			break;
 		case 'F':
 			freeze = 1;
 			break;
@@ -249,6 +277,9 @@ main(argc, argv)
 			version = 1;
 			daemonflg = 0;
 			break;
+		case 'S':
+			syncstart = 1;
+			break;
 		case '?':
 		default:
 			errflg++;
@@ -397,6 +428,23 @@ main(argc, argv)
 #ifdef	MALLOC_TRACE
 	mal_leaktrace(1);
 #endif	/* MALLOC_TRACE */
+
+	/* If we do a sync-start, we are synchronous.. */
+
+	if (syncstart) {
+	  int startcount;
+	  dirqueuescan(".", dq);
+	  startcount = dq->wrkcount;
+	  while (dq->wrkcount > 0) {
+	    syncweb(dq, tp);
+	    queryipccheck();
+	  }
+	  fprintf(stderr,"Synchronous startup completed, messages: %d (%d skipped) recipients: %d\n",
+		  global_wrkcnt, startcount - global_wrkcnt, thread_count_recipients());
+	  fprintf(stderr,"***********************************************************************\n");
+	  syncstart = 0;
+	}
+
 	do {
 	  time_t timeout;
 
@@ -438,7 +486,7 @@ main(argc, argv)
 	    timeout = now;
 
 	  /* Submit possible new jobs (unless frozen) */
-	  if (!freeze && doagenda() != 0)
+	  if (!freeze && !syncstart && doagenda() != 0)
 	    timeout = now;	/* we still have some jobs avail for start */
 
 	  gotalarm = 0;
@@ -526,9 +574,8 @@ static int in_dirscanqueue(dq,ino)
 	return 0;
 }
 
-static int dq_ctimecompare(a,b)
-const void *a;
-const void *b;
+static int dq_ctimecompare(b,a) /* we want oldest entry LAST */
+const void *a; const void *b;
 {
 	const struct dirstatname **A = (const struct dirstatname **)a;
 	const struct dirstatname **B = (const struct dirstatname **)b;
@@ -566,7 +613,8 @@ if (verbose) { printf("dirqueuescan(dir='%s') ",dir); fflush(stdout); }
 	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
 	  /* Scan filenames into memory */
 
-	  if (newents > 200) break; /* At most 200 per one go */
+	  if (!syncstart && newents > newents_limit)
+	    break; /* At most XXX per one go */
 
 	  if (dp->d_name[0] >= '0' &&
 	      dp->d_name[0] <= '9') {
@@ -622,8 +670,9 @@ if (verbose) { printf("dirqueuescan(dir='%s') ",dir); fflush(stdout); }
 	    ++newents;
 	  } /* ... end of "filename starts with a [0-9]" */
 	}
-	if (newents) {
-	  /* Sort the dq->stats[] per file ctime -- last on slot 0. */
+	if (newents && dq->wrkcount > 1) {
+	  /* Sort the dq->stats[] per file ctime -- LATEST on slot 0.
+	     (we drain this queue from the END) */
 	  qsort((void*)dq->stats,
 		dq->wrkcount,
 		sizeof(void*),
@@ -668,7 +717,8 @@ static int syncweb(dq, cehead)
 	ino   =   dq->stats[dq->wrkcount]->ino;
 	/* Now we have pointers */
 
-	sp_delete(sp_lookup(ino,dirscan_mesh),dirscan_mesh); /* XX: should ALWAYS succeed.. */
+	/* Deletion from the  dirscan_mesh  should ALWAYS succeed.. */
+	sp_delete(sp_lookup(ino,dirscan_mesh),dirscan_mesh);
 
 	/* Sometimes we may get files already in processing
 	   into our pre-schedule queue */
@@ -685,9 +735,11 @@ static int syncweb(dq, cehead)
 	      eunlink(file);	/* hrmpf! */
 	  } else {
 	    /* Ok, schedule! */
-	    schedule(fd, file, cehead, ino);
-	    ++wrkcnt;
-	    ++global_wrkcnt;
+	    if (schedule(fd, file, cehead, ino) != NULL) {
+	      /* Success, increment counters */
+	      ++wrkcnt;
+	      ++global_wrkcnt;
+	    }
 	  }
 	}
 
@@ -727,6 +779,7 @@ static struct ctlfile *schedule(fd, file, cehead, ino)
 	  return NULL;
 	}
 	if (cfp->head == NULL) {
+	  ++global_wrkcnt;
 	  unctlfile(cfp, 0);
 	  return NULL;
 	}
@@ -735,6 +788,14 @@ static struct ctlfile *schedule(fd, file, cehead, ino)
 	  if (cfp->vfp)
 	    fseek(cfp->vfp, 0, 2);
 	}
+	/* If no verbose trace, don't keep contents in core!
+	   this means HUGE memory save at some machines! */
+	if (cfp->vfp == NULL)
+	  if (cfp->contents != NULL) {
+	    free(cfp->contents);
+	    cfp->contents = NULL;
+	  }
+
 	return cfp;
 }
 
@@ -777,17 +838,20 @@ slurp(fd, ino)
 	memset((void*)cfp, 0, sizeof(struct ctlfile));
 
 	cfp->fd = fd;
-	cfp->vfp = NULL;
-	cfp->head = NULL;
 	cfp->uid = stbuf.st_uid;
 	cfp->envctime = stbuf.st_ctime;
-	cfp->mark = V_NONE;
-	cfp->haderror = 0;
 	cfp->contents = contents;
-	cfp->envid    = NULL;
-	cfp->logident = NULL;
-	cfp->erroraddr = NULL;
-	cfp->msgbodyoffset = 0;
+	/* 
+	   cfp->vfp = NULL;
+	   cfp->head = NULL;
+	   cfp->mark = V_NONE;
+	   cfp->haderror = 0;
+	   cfp->mid      = NULL;
+	   cfp->envid    = NULL;
+	   cfp->logident = NULL;
+	   cfp->erroraddr = NULL;
+	   cfp->msgbodyoffset = 0;
+	 */
 
 	/* go through the file and mark it off */
 	i = 0;
@@ -813,16 +877,17 @@ slurp(fd, ino)
 		if (s - contents >= stbuf.st_size) {
 		  /* XX: header ran off file */
 		}
-	      } else if (*s == _CF_BODYOFFSET)
+	      } else if (*s == _CF_BODYOFFSET) {
 		cfp->msgbodyoffset = atoi(s+2);
-	      else if (*s == _CF_DSNENVID)
-		cfp->envid = s+2;
-	      else if (*s == _CF_MESSAGEID)
-		cfp->mid = s+2;
-	      else if (*s == _CF_LOGIDENT)
-		cfp->logident = s+2;
-	      else if (*s == _CF_ERRORADDR)
-		cfp->erroraddr = s+2;
+	      } else if (*s == _CF_DSNENVID) {
+		/* cfp->envid = strsave(s+2); */
+	      } else if (*s == _CF_MESSAGEID) {
+		/* cfp->mid = strsave(s+2); */
+	      } else if (*s == _CF_LOGIDENT) {
+		/* cfp->logident = strsave(s+2); */
+	      } else if (*s == _CF_ERRORADDR) {
+		/* cfp->erroraddr = strsave(s+2); */
+	      }
 	    }
 	  }
 	}
@@ -837,7 +902,7 @@ slurp(fd, ino)
 	while (--i >= 0)
 	  *lp++ = *ip++;
 	cfp->id = ino;
-	cfp->mid = NULL;
+	/* cfp->mid = NULL; */
 	free(offset);	/* release the block */
 	return cfp;
 }
@@ -959,6 +1024,12 @@ static struct ctlfile *vtxprep(cfp)
 	    lockaddr(cfp->fd, (long) (cp - cfp->contents),
 		     _CFTAG_LOCK, _CFTAG_NORMAL);
 	  }
+	  /* Calculate summary info */
+	  if (cp[-1] == _CF_RECIPIENT) {
+	    cfp->rcpnts_total += 1;
+	    if (*cp == _CFTAG_NOTOK)   cfp->rcpnts_failed += 1;
+	    else if (*cp == _CFTAG_OK) cfp->rcpnts_work   += 1;
+	  }
 	  if (*cp == _CFTAG_NORMAL ||
 	      *cp == '\n' /* This appears for msg-header entries.. */ ) {
 	    --cp;
@@ -970,7 +1041,8 @@ static struct ctlfile *vtxprep(cfp)
 	      while (*cp != 0 && *cp != ' ') ++cp; while (*cp == ' ') ++cp;
 	      latest_sender = cp;
 	      while (*cp != 0 && *cp != ' ') ++cp;  *cp = 0; /* We assume: No spaces at the sender addr! */
-	      cfp->erroraddr = latest_sender;
+	      if (cfp->erroraddr)  free(cfp->erroraddr);
+	      cfp->erroraddr = strsave(latest_sender);
 	      break;
 	    case _CF_RECIPIENT:
 	      if (opcnt >= offspc-1) {
@@ -989,12 +1061,8 @@ static struct ctlfile *vtxprep(cfp)
 	      offarr[opcnt].headeroffset = -1;
 	      offarr[opcnt].drptoffset = -1;
 	      offarr[opcnt].sender = latest_sender;
-	      offarr[opcnt].dsnrecipient = NULL;
 	      ++opcnt;
 	      break;
-	    case _CF_DSNRECIPIENT:
-	      offarr[svn].dsnrecipient = cp+2;
-	      break;
 	    case _CF_RCPTNOTARY:
 	      /* XX: IETF-NOTARY-DRPT DATA! */
 	      offarr[svn].drptoffset = *lp + 2;
@@ -1052,22 +1120,26 @@ static struct ctlfile *vtxprep(cfp)
 		offarr[svn].headeroffset = *lp + 2;
 	      break;
 	    case _CF_DSNENVID:
-	      cfp->envid = cp+2;
+	      if (cfp->envid) free(cfp->envid); /* shouldn't happen.. */
+	      cfp->envid = strsave(cp+2);
 	      break;
 	    case _CF_DIAGNOSTIC:
 	      cfp->haderror = 1;
 	      break;
 	    case _CF_MESSAGEID:
-	      cfp->mid = cp+2;
+	      if (cfp->mid != NULL) free(cfp->mid); /* shouldn't happen.. */
+	      cfp->mid = strsave(cp+2);
 	      break;
 	    case _CF_BODYOFFSET:
 	      cfp->msgbodyoffset = atoi(cp+2);
 	      break;
 	    case _CF_LOGIDENT:
-	      cfp->logident = cp+2;
+	      if (cfp->logident) free(cfp->logident); /* shouldn't happen..*/
+	      cfp->logident = strsave(cp+2);
 	      break;
 	    case _CF_ERRORADDR:
-	      cfp->erroraddr = cp+2;
+	      if (cfp->erroraddr) free(cfp->erroraddr); /* could happen */
+	      cfp->erroraddr = strsave(cp+2);
 	      break;
 	    case _CF_OBSOLETES:
 	      deletemsg(cp+2, cfp);
@@ -1120,7 +1192,12 @@ static struct ctlfile *vtxprep(cfp)
 	  free(offarr);
 	  return NULL;
 	}
-	cfp->ctime = stbuf.st_ctime;
+	cfp->ctime = stbuf.st_mtime; /* instead of ctime, we use mtime
+					at the queue-dir accesses, this
+					way we can move the spool to
+					another machine, and run things
+					in there, and still have same
+					expiration times.. */
 	cfp->fd = -1;
 	/* sort it to get channels and channel/hosts together */
 	bcp = cfp->contents;
@@ -1129,8 +1206,7 @@ static struct ctlfile *vtxprep(cfp)
 	 * Loop through them; whenever either channel or host changes,
 	 * make a new vertex. All such vertices must be linked together.
 	 */
-	flagstr[0] = ' ';
-	flagstr[1] = '\0';
+	strcpy(flagstr," ");
 	l_channel = l_echannel = l_host = l_ehost = flagstr;
 	svn = 0;
 	pvp = NULL;
@@ -1141,6 +1217,7 @@ static struct ctlfile *vtxprep(cfp)
 	  if ((echannel = strchr(channel, ' ')) == NULL) /* error! */
 	    continue;
 	  *echannel = '\0';
+	  strlower(channel);
 	  host = echannel + 1;
 	  while (*host == ' ') ++host;
 #if 1
@@ -1177,8 +1254,9 @@ static struct ctlfile *vtxprep(cfp)
 	      continue;
 
 	  *ehost = '\0';
+	  strlower(host);
 	  /* compare with the last ones */
-	  if (cistrcmp(channel, l_channel) || cistrcmp(host, l_host)) {
+	  if (strcmp(channel, l_channel) || strcmp(host, l_host)) {
 	    /* wrap and tie the old vertex node */
 	    if (i != svn) {
 	      u_int alloc_size = (u_int) (sizeof (struct vertex) +
@@ -1186,20 +1264,20 @@ static struct ctlfile *vtxprep(cfp)
 	      vp = (struct vertex *)emalloc(alloc_size);
 	      memset((char*)vp, 0, alloc_size);
 	      vp->cfp          = cfp;
-	      vp->linkcnt      = 0;
-	      defaultconfigentry(&vp->ce,NULL);
 	      vp->next[L_CTLFILE] = NULL;
 	      vp->prev[L_CTLFILE] = pvp;
+#if 0
 	      vp->message      = NULL;
 	      vp->retryindex   = 0;
 	      vp->nextitem     = NULL;
 	      vp->previtem     = NULL;
 	      vp->wakeup       = 0;
 	      vp->proc         = NULL;
-	      vp->ngroup       = i - svn;
 	      vp->attempts     = 0;
 	      vp->notary       = NULL;
-	      vp->sender       = offarr[svn].sender;
+#endif
+	      vp->ngroup       = i - svn;
+	      /* vp->sender       = strsave(offarr[svn].sender); */
 	      vp->headeroffset = offarr[svn].headeroffset; /*They are similar*/
 	      vp->drptoffset   = offarr[svn].drptoffset;
 	      while (svn < i) {
@@ -1231,20 +1309,20 @@ static struct ctlfile *vtxprep(cfp)
 	  vp = (struct vertex *)emalloc(alloc_size);
 	  memset((void*)vp, 0, alloc_size);
 	  vp->cfp = cfp;
-	  vp->linkcnt  = 0;
-	  defaultconfigentry(&vp->ce,NULL);
 	  vp->next[L_CTLFILE] = NULL;
 	  vp->prev[L_CTLFILE] = pvp;
+#if 0
 	  vp->message = NULL;
 	  vp->retryindex = 0;
 	  vp->nextitem = NULL;
 	  vp->previtem = NULL;
 	  vp->proc = NULL;
 	  vp->wakeup = 0;
+	  vp->attempts     = 0;
+	  vp->notary       = NULL;
+#endif
 	  vp->ngroup = i - svn;
-	  vp->attempts = 0;
-	  vp->notary = NULL;
-	  vp->sender = offarr[i].sender;
+	  /* vp->sender = strsave(offarr[snv].sender); */
 	  vp->headeroffset = offarr[svn].headeroffset; /* Just any of them will do */
 	  vp->drptoffset = offarr[svn].drptoffset;
 	  while (svn < i) {
@@ -1272,12 +1350,12 @@ static struct ctlfile *vtxprep(cfp)
 }
 
 /*
- *  The  vtxfill()  is a subroutine to  vtxdo()  taking care of filling in
- *  scheduler definition entries from scheduler configuration table.
+ *  The  vtxmatch()  is a subroutine to  vtxdo()  matching for
+ *  scheduler definition entries from the scheduler configuration table.
  *
  */
 
-static int vtxfill(vp, tp)
+static int vtxmatch(vp, tp)
 	struct vertex *vp;
 	struct config_entry *tp;
 {
@@ -1298,39 +1376,52 @@ static int vtxfill(vp, tp)
 	if (verbose>1)
 	  printf("host %s %s\n", vp->orig[L_HOST]->name, tp->host);
 
-	if (tp->interval != -1) vp->ce.interval = tp->interval;
-	if (tp->idlemax  != -1) vp->ce.idlemax  = tp->idlemax;
-	if (tp->expiry   != -1) vp->ce.expiry   = tp->expiry;
-	vp->ce.expiryform     = tp->expiryform;
-	vp->cfp->deliveryform = tp->deliveryform;
-	if (tp->uid      != -1) vp->ce.uid = tp->uid;
-	if (tp->gid      != -1) vp->ce.gid = tp->gid;
-	if (tp->maxkids  != -1) vp->ce.maxkids = tp->maxkids;
-	if (tp->maxkidChannel != -1) vp->ce.maxkidChannel = tp->maxkidChannel;
-	if (tp->maxkidThreads != -1) vp->ce.maxkidThreads = tp->maxkidThreads;
-	if (tp->nretries > 0) {
-	  vp->ce.nretries = tp->nretries;
-	  vp->ce.retries  = tp->retries;
-	}
-	if (tp->command != NULL) {
-	  vp->ce.command = tp->command;
-	  vp->ce.argv    = tp->argv;
-	}
-	if (tp->bychannel != -1)
-	  vp->ce.bychannel = tp->bychannel;
-	else
-	  vp->ce.bychannel = 0;
-	if (tp->withhost != -1)
-	  vp->ce.withhost = tp->withhost;
-	else
-	  vp->ce.withhost = 0;
-	vp->ce.flags |= tp->flags;
-	vp->ce.mark   = vp->ce.pending = 0;
-	vp->ce.host   = tp->host;
-	if (tp->skew > 0) vp->ce.skew = tp->skew;
 	return 1;
 }
 
+
+static void ce_fillin(thg,cep)
+	struct threadgroup *thg;
+	struct config_entry *cep;
+{
+	struct config_entry *ce = &(thg->ce);
+
+	defaultconfigentry( ce,cep );
+
+	if (cep->interval != -1) ce->interval = cep->interval;
+	if (cep->idlemax  != -1) ce->idlemax  = cep->idlemax;
+	if (cep->expiry   != -1) ce->expiry   = cep->expiry;
+	ce->expiryform     = cep->expiryform;
+	if (cep->uid      != -1) ce->uid = cep->uid;
+	if (cep->gid      != -1) ce->gid = cep->gid;
+	if (cep->maxkids  != -1) ce->maxkids = cep->maxkids;
+	if (cep->maxkidChannel != -1) ce->maxkidChannel = cep->maxkidChannel;
+	if (cep->maxkidThreads != -1) ce->maxkidThreads = cep->maxkidThreads;
+	if (cep->nretries > 0) {
+	  ce->nretries = cep->nretries;
+	  ce->retries  = cep->retries;
+	}
+	if (cep->command != NULL) {
+	  ce->command = cep->command;
+	  ce->argv    = cep->argv;
+	}
+	ce->flags |= cep->flags; /* XX: Grumble.. additive only.. */
+	ce->host   = cep->host;
+	if (cep->skew > 0) ce->skew = cep->skew;
+
+	if (ce->interval == -1)
+	  ce->interval = 3600;
+	if (ce->idlemax == -1)
+	  ce->idlemax = ce->interval * 3;
+	if (ce->maxkids == -1)
+	  ce->maxkids = global_maxkids;
+	if (ce->maxkidChannel == -1)
+	  ce->maxkidChannel = global_maxkids;
+	if (ce->maxkidThreads == -1)
+	  ce->maxkidThreads = global_maxkids;
+}
+
+
 /*
  *  The  vtxdo()  tries thru all scheduler configuration entries
  *  looking for a matching one which to fill in for the input vertex.
@@ -1356,9 +1447,11 @@ static void vtxdo(vp, cehead, path)
 	n = 0;
 	for (tp = cehead; tp != NULL; tp = tp->next) {
 	  ++cnt;
-	  n += vtxfill(vp, tp);
-	  if (vp->ce.command != NULL)
+	  if (vtxmatch(vp, tp)) {
+	    /* tp points to the selected config entry */
+	    n = 1;
 	    break;
+	  }
 	}
 	if (n == 0) {
 	  fprintf(stderr, "%s: no pattern matched %s/%s address\n",
@@ -1371,26 +1464,12 @@ static void vtxdo(vp, cehead, path)
 		 vp->orig[L_CHANNEL]->name, vp->orig[L_HOST]->name);
 
 	/* set default values */
-	if (vp->ce.expiry > 0)
-	  vp->ce.expiry += vp->cfp->ctime;
+	if (tp->expiry > 0)
+	  vp->ce_expiry = tp->expiry + vp->cfp->ctime;
 	else
-	  vp->ce.expiry = 0;
-	if (vp->ce.interval == -1)
-	  vp->ce.interval = 3600;
-	if (vp->ce.idlemax == -1)
-	  vp->ce.idlemax = vp->ce.interval * 3;
-	if (vp->ce.maxkids == -1)
-	  vp->ce.maxkids = global_maxkids;
-	if (vp->ce.maxkidChannel == -1)
-	  vp->ce.maxkidChannel = global_maxkids;
-	if (vp->ce.maxkidThreads == -1)
-	  vp->ce.maxkidThreads = global_maxkids;
-	if (vp->ce.withhost == -1)
-	  vp->ce.withhost = 0;
-	if (vp->ce.bychannel == -1)
-	  vp->ce.bychannel = 0;
+	  vp->ce_expiry = 0;
 
-	thread_linkin(vp,tp,cnt);
+	thread_linkin(vp,tp,cnt,ce_fillin);
 
 	if (verbose>1)
 	  vtxprint(vp);
@@ -1405,11 +1484,7 @@ int vtxredo(spl)
 
         /* assert cfp != NULL */
 	for (vp = cfp->head ; vp != NULL ; vp = vp->next[L_CTLFILE]) {
-		vp->ce.command = NULL;
-		vp->ce.argv = NULL;
-		vp->ce.retries = NULL;
-		vp->ce.nretries = 0;
-		vtxdo(vp, rrcf_head, NULL);
+	  vtxdo(vp, rrcf_head, NULL);
 	}
         return 0;
 }
@@ -1424,6 +1499,30 @@ static int globmatch(pattern, string)
 	  switch (*pattern) {
 	  case '*':
 	    ++pattern;
+	    if (*pattern == 0) {
+	      /* pattern ended with '*', we can accept any string trail.. */
+	      return 1;
+	    }
+	    /* We do 'common case' optimization here, but will loose some
+	       performance, if somebody gives '*foo*' as a pattern.. */
+	    {
+	      char *p = pattern;
+	      int i = 0, c;
+	      while ((c = *p++)) {
+		/* Scan for special chars in pattern.. */
+		if (c == '*' || c == '[' || c == '\\' || c == '?') {
+		  i = 1; /* Found! */
+		  break;
+		}
+	      }
+	      if (!i) { /* No specials, match from end of string */
+		int len = strlen(string);
+		i = strlen(pattern);
+		if (i > len) return 0; /* Tough.. pattern longer than string */
+		if (strcmp(string +(len-i),pattern) == 0)
+		  return 1; /* MATCH! */
+	      }
+	    }
 	    do {
 	      if (globmatch(pattern, string))
 		return 1;
@@ -1500,7 +1599,7 @@ static void link_in(flag, vp, s)
 }
 
 /*
- * time-string to be prepended on logged messages
+ * time-string to be prepended to logged messages
  */
 char *
 timestring()
