@@ -28,12 +28,14 @@ struct thread      *thread_tail = NULL;
 static struct threadgroup *thrg_root   = NULL;
 int idleprocs = 0;
 extern int global_wrkcnt;
+extern int syncstart;
 
 struct threadgroup *
-create_threadgroup(cep, wc, wh, withhost)
+create_threadgroup(cep, wc, wh, withhost, ce_fillin)
 struct config_entry *cep;
 struct web *wc, *wh;
 int withhost;
+void (*ce_fillin) __((struct threadgroup*, struct config_entry *));
 {
 	struct threadgroup *thgp;
 
@@ -48,6 +50,8 @@ int withhost;
 	thgp->wchan    = wc;
 	thgp->whost    = wh;
 
+	ce_fillin(thgp,cep);
+
 	wc->linkcnt += 1;
 	wh->linkcnt += 1;
 
@@ -141,10 +145,16 @@ struct thread *thr;
 	thread_tail = thr;
 }
 
-struct thread *
-create_thread(thgrp, vtx)
+
+static struct thread *create_thread __((struct threadgroup *thgrp,
+					struct vertex *vtx,
+					struct config_entry *cep));
+
+static struct thread *
+create_thread(thgrp, vtx, cep)
 struct threadgroup *thgrp;
 struct vertex *vtx;
+struct config_entry *cep;
 {
 	/* Create a thread-block, link in the group pointer,
 	   and link the thread into thread-ring, plus APPEND
@@ -156,9 +166,11 @@ struct vertex *vtx;
 
 	memset(thr,0,sizeof(*thr));
 
-	thr->thgrp = thgrp;
+	thr->thgrp   = thgrp;
 	thr->nextthg = thr;
 	thr->prevthg = thr;
+	thr->wchan   = vtx->orig[L_CHANNEL];
+	thr->whost   = vtx->orig[L_HOST];
 
 	if (thgrp->thread == NULL) {
 	  thgrp->thread = thr;
@@ -173,11 +185,11 @@ struct vertex *vtx;
 	thr->lastvertex = vtx;
 	thr->jobs       = 1;
 	vtx->thread     = thr;
-	thr->channel    = strdup(vtx->orig[L_CHANNEL]->name);
-	thr->host       = strdup(vtx->orig[L_HOST   ]->name);
+	thr->channel    = strsave(vtx->orig[L_CHANNEL]->name);
+	thr->host       = strsave(vtx->orig[L_HOST   ]->name);
 
-	if (verbose) printf("create_thread(%s/%d/%s) -> 0x%lx\n",
-			    thr->channel,thgrp->withhost,thr->host,(long)thr);
+	if (verbose) printf("create_thread(%s/%d/%s) -> 0x%p\n",
+			    thr->channel,thgrp->withhost,thr->host,thr);
 
 	_thread_timechain_append(thr);
 
@@ -196,8 +208,8 @@ int ok;
 	int last = 0;
 
 	if (verbose)
-	  printf("delete_thread(0x%lx:%s/%s) (thg=0x%lx)\n",
-		 (long)thr,thr->channel,thr->host,(long)thg);
+	  printf("delete_thread(0x%p:%s/%s) (thg=0x%p)\n",
+		 thr,thr->channel,thr->host,thg);
 
 	free(thr->channel);
 	free(thr->host);
@@ -240,12 +252,12 @@ int ok;
 	    thr->proc->thread = NULL;
 	    thr->proc->vertex = NULL;
 	    if (verbose)
-	      fprintf(stderr,"delete_thread(1) thr->proc=0x%lx pid=%d\n",
-		      (long)thr->proc, thr->proc->pid);
+	      fprintf(stderr,"delete_thread(1) thr->proc=0x%p pid=%d\n",
+		      thr->proc, thr->proc->pid);
 	  } else {
 	    if (verbose)
-	      fprintf(stderr,"delete_thread(1b) thr->proc=0x%lx pid=%d\n",
-		      (long)thr->proc, thr->proc ? thr->proc->pid : 0);
+	      fprintf(stderr,"delete_thread(1b) thr->proc=0x%p pid=%d\n",
+		      thr->proc, thr->proc ? thr->proc->pid : 0);
 	  }
 	} else {
 	  /* Some threads left						*/
@@ -253,7 +265,7 @@ int ok;
 	     the process' current thread is no longer in the ring!	*/
 	  if (thr->proc &&
 	      thr->proc->thread == thr) {
-	    struct thread *thr0, *thr1;
+	    struct thread *thr0, *thr1, *thrn;
 	    struct procinfo *proc = thr->proc;
 	    int once = 1;
 	    /* Move it into the idle pool -- and try to find
@@ -265,41 +277,36 @@ int ok;
 	    thg->idlecnt += 1;
 	    ++idleprocs;
 	    if (verbose)
-	      fprintf(stderr, "delete_thread(2) thr->proc=0x%lx pid=%d ",
-		      (long)thr->proc, thr->proc->pid);
+	      fprintf(stderr, "delete_thread(2) thr->proc=0x%p pid=%d ",
+		      thr->proc, thr->proc->pid);
 	    /* Find a free thread - or stay in idle.. */
 	    for (thr0 = thg->thread, thr1 = thr0;
 		 once || thr0 != thr1;
-		 thr1 = thr1->nextthg) {
+		 thr1 = thrn) {
+	      /* Attempt to  thread_start() may scramble the thr1 object.. */
+	      thrn = thr1->nextthg;
 	      once = 0;
 	      if (thr1->proc == NULL ||
 		  thr1->proc->thread != thr1) {
 		/* Ok, this thread is not busy, choose it! */
-		proc->thread = thr1;
-		proc->vertex = thr1->vertices;
-		thr1->proc = proc;
-		/* Contrary to the previous guess, we are not idle! */
-		thg->idleproc = proc->next;
-		thg->idlecnt -= 1;
-		--idleprocs;
-		pick_next_vertex(proc, ok);
+		thread_start(thr1);
 		break;
 	      }
 	    }
 	    if (verbose)
-	      fprintf(stderr,"proc->thr=0x%x\n",(long)proc->thread);
+	      fprintf(stderr,"proc->thr=0x%p\n",proc->thread);
 
 	  } else if (thr->proc != NULL &&
 		     thr->proc->pid <= 0) {
 	    thr->proc->thread = NULL;
 	    thr->proc->vertex = NULL;
 	    if (verbose)
-	      fprintf(stderr, "delete_thread(2b) thr->proc=0x%lx pid=%d\n",
-		      (long)thr->proc, thr->proc->pid);
+	      fprintf(stderr, "delete_thread(2b) thr->proc=0x%p pid=%d\n",
+		      thr->proc, thr->proc->pid);
 	  } else {
 	    if (verbose)
-	      fprintf(stderr, "delete_thread(3) thr->proc=0x%lx pid=%d\n", (long)thr->proc,
-		      thr->proc ? thr->proc->pid : 0);
+	      fprintf(stderr, "delete_thread(3) thr->proc=0x%p pid=%d\n",
+		      thr->proc, thr->proc ? thr->proc->pid : 0);
 	  }
 	}
 
@@ -318,6 +325,7 @@ struct vertex *ap, *vp;
 	vp->nextitem = ap;
 	if (ap == thr->vertices)
 	  thr->vertices = vp;
+	vp->thread = thr;
 }
 
 
@@ -338,10 +346,11 @@ struct vertex *vp;
 	thr->lastvertex = vp;
 }
 
-void thread_linkin(vp,cep,cfgid)
+void thread_linkin(vp,cep,cfgid, ce_fillin)
 struct vertex *vp;
 struct config_entry *cep;
 int cfgid;
+void (*ce_fillin) __((struct threadgroup*, struct config_entry *));
 {
 	struct threadgroup *thg;
 	struct thread *thr;
@@ -353,13 +362,14 @@ int cfgid;
 	struct web *wh = vp->orig[L_HOST];
 
 if (verbose) printf("thread_linkin([%s/%s],%s/%d/%s,%d)\n",wc->name,wh->name,
-		    cep->channel,cep->withhost,cep->host,cfgid);
+		    cep->channel,cep->flags & CFG_WITHHOST,cep->host,cfgid);
 
 	/* char const *vp_chan = wc->name; */
 	/* char const *vp_host = wh->name; */
 
-	if (thrg_root == NULL)
-	  (void) create_threadgroup(cep, wc, wh, cep->withhost);
+	if (thrg_root == NULL) {
+	  (void) create_threadgroup(cep, wc, wh, cep->flags & CFG_WITHHOST, ce_fillin);
+	}
 
 	/*
 	 *  Search for matching config-entry, AND matching channel,
@@ -393,57 +403,68 @@ if (verbose) printf("thread_linkin([%s/%s],%s/%d/%s,%d)\n",wc->name,wh->name,
 	       thr != NULL && (thr_once || thr != thg->thread);
 	       thr = thr->nextthg) {
 
-	    struct vertex *vt = thr->vertices;
-
 	    thr_once = 0;
-
-	    if (!vt) abort();	/* No vertices ?? */
+#if 0
+	    if (!thr->vertex) abort();	/* No vertices ?? */
 
 	    /* no need ? (Channels within a group are identical..) */
-	    if (wc != vt->orig[L_CHANNEL])  abort();
-
+	    if (wc != thr->wchan)  abort();
+#endif
 	    /* Nice! What about host ? */
-	    if (wh != vt->orig[L_HOST])     continue;
+	    if (wh != thr->whost)  continue;
 	    
 	    /* We have matching channel, AND matching host */
 
 	    /* Link the vertex into this thread! */
 
-if (verbose) printf("thread_linkin() to thg=0x%lx[%s/%d/%s]; added into existing thread [%s/%s] thr->jobs=%d\n",
-		    (long)thg,cep->channel,thg->withhost,cep->host,
+if (verbose) printf("thread_linkin() to thg=0x%p[%s/%d/%s]; added into existing thread [%s/%s] thr->jobs=%d\n",
+		    thg,cep->channel,thg->withhost,cep->host,
 		    wc->name,wh->name,thr->jobs+1);
 
 	    _thread_linktail(thr,vp);
+	    vp->thgrp = thg;
 	    thr->jobs += 1;
-	    /* Hookay..  Try to start it, in case it isn't yet runing */
+	    /* Hookay..  Try to start it, in case it isn't yet running */
 	    thread_start(thr);
 
 	    return;
 	  }
 	  /* No matching thread, however this GROUP matches (or does it?) */
 
-if (verbose) printf("thread_linkin() to thg=0x%lx[%s/%d/%s]; created a new thread [%s/%s]\n",
-		    (long)thg,cep->channel,thg->withhost,cep->host,
+if (verbose) printf("thread_linkin() to thg=0x%p[%s/%d/%s]; created a new thread [%s/%s]\n",
+		    thg,cep->channel,thg->withhost,cep->host,
 		    wc->name,wh->name);
 
 	  /* Add a new thread into this group */
-	  thr = create_thread(thg,vp);
+	  thr = create_thread(thg,vp,cep);
+	  vp->thgrp = thg;
 	  /* Try to start it too */
 	  thread_start(thr);
 	  return;
 	}
 
 	/* Add a new group - and its thread .. */
-	thg = create_threadgroup(cep, wc, wh, cep->withhost);
-	thr = create_thread(thg,vp);
+	thg = create_threadgroup(cep, wc, wh, cep->flags & CFG_WITHHOST, ce_fillin);
+	thr = create_thread(thg,vp,cep);
+	vp->thgrp = thg;
 
-if (verbose) printf("thread_linkin() to thg=0x%lx[%s/%d/%s]; created a new thread group, and thread [%s/%s]\n",
-		    (long)thg,cep->channel,thg->withhost,cep->host,
+if (verbose) printf("thread_linkin() to thg=0x%p[%s/%d/%s]; created a new thread group, and thread [%s/%s]\n",
+		    thg,cep->channel,thg->withhost,cep->host,
 		    wc->name,wh->name);
 	/* Try to start it too */
 	thread_start(thr);
 }
 
+char *strlower(s)
+char *s;
+{
+	char *cp;
+	for (cp = s; *cp != '\0'; ++cp)
+	  if (isascii(*cp) && isupper(*cp))
+	    *cp = tolower(*cp);
+	return s;
+}
+
 struct web *
 web_findcreate(flag, s)
 int flag;
@@ -453,9 +474,7 @@ u_char *s;
 	register u_char *cp;
 	struct web *wp;
 
-	for (cp = s; *cp != '\0'; ++cp)
-	  if (isascii(*cp) && isupper(*cp))
-	    *cp = tolower(*cp);
+	/* caller has done 'strlower()' to our input.. */
 
 	spl = sp_lookup(symbol(s), spt_mesh[flag]);
 	if (spl == NULL || (wp = (struct web *)spl->data) == NULL) {
@@ -557,6 +576,20 @@ web_disentangle(vp, ok)
 	  delete_thread(vp->thread, ok);
 }
 
+static int vtx_ctime_cmp(a, b)
+struct vertex **a, **b;
+{
+	if ((*a)->cfp->ctime < (*b)->cfp->ctime)
+	  return -1;
+	else if ((*a)->cfp->ctime == (*b)->cfp->ctime)
+	  /* If same age, order in memory defines the result,
+	     and thus stabilizing the sort result.            */
+	  return (*a)->cfp - (*b)->cfp;
+	else
+	  return 1;
+}
+
+
 void
 thread_vertex_shuffle(thr)
 struct thread *thr;
@@ -566,7 +599,9 @@ struct thread *thr;
 	static u_int           ur_size = 0;
 	static struct vertex **ur_arr  = NULL;
 
-	/* Randomize the order of vertices in processing  */
+	/* Randomize the order of vertices in processing, OR
+	   sort them by spool-file CTIME, if the thread has
+	   AGEORDER -flag set. */
 
 	/* 1) count the number of elts -- changed so it will be done
 	      while creating the storage array.. */
@@ -587,13 +622,19 @@ struct thread *thr;
 	  }
 	  ur_arr[n++] = vp;
 	}
+
 	/* 4) re-arrange pointers */
-	for (i = 0; i < n; ++i) {
-	  ni = ranny(n-1);
-	  vp = ur_arr[i];
-	  ur_arr[i] = ur_arr[ni];
-	  ur_arr[ni] = vp;
-	}
+	if (thr->thgrp->ce.flags & CFG_AGEORDER) {
+	  /* ctime order */
+	  qsort((void*)ur_arr, n, sizeof(struct vertex *), vtx_ctime_cmp);
+	} else
+	  /* Random order */
+	  for (i = 0; i < n; ++i) {
+	    ni = ranny(n-1);
+	    vp = ur_arr[i];
+	    ur_arr[i] = ur_arr[ni];
+	    ur_arr[ni] = vp;
+	  }
 	/* 5) Relink previtem/nextitem pointers */
 	for (i = 0; i < n; ++i) {
 	  if (i > 0)
@@ -629,12 +670,14 @@ struct thread *thr;
 	int rc;
 	struct vertex      *vp  = thr->vertices;
 	struct threadgroup *thg = thr->thgrp;
+	struct config_entry *ce = &(thr->thgrp->ce);
+
+	if (syncstart) return 0;
 
 	if (verbose)
-	  fprintf(stderr,"thread_start(thr=%s/%d/%s) (dt=%d, thr=0x%x)",
+	  fprintf(stderr,"thread_start(thr=%s/%d/%s) (dt=%d, thr=0x%p)\n",
 		  vp->orig[L_CHANNEL]->name, thg->withhost,
-		  vp->orig[L_HOST]->name, (int)(thr->wakeup-now),
-		  (long)thr);
+		  vp->orig[L_HOST]->name, (int)(thr->wakeup-now),thr);
 
 	if (thr->proc != NULL &&
 	    thr->proc->thread == thr) {
@@ -643,6 +686,7 @@ struct thread *thr;
 	} else
 	  if (verbose) fprintf(stderr,"\n");
 
+      re_pick:
 	if (thg->idleproc != NULL) {
 	  struct procinfo *proc;
 
@@ -656,6 +700,10 @@ struct thread *thr;
 	  thg->idlecnt -= 1;
 	  --idleprocs;
 	  proc->next    = NULL;
+	  
+	  /* It may be that while we idled it, it died at the idle queue.. */
+	  if (proc->pid <= 0 || proc->tofd < 0)
+	    goto re_pick;
 
 	  proc->thread  = thr;
 	  proc->vertex  = thr->vertices;
@@ -674,12 +722,39 @@ struct thread *thr;
 	  proc->ho = thr->vertices->orig[L_HOST];
 	  proc->ho->kids += 1;
 
-	  /* Clean vertices 'proc'-pointers,
-	     Randomize the order of thread vertices */
+	  /* Clean vertices 'proc'-pointers,  randomize the
+	     order of thread vertices  (or sort by spool file
+	     ctime, if in AGEORDER..) */
 	  thread_vertex_shuffle(thr);
 
+	  thr->attempts += 1;
+
+	  proc->fed = 0;
 	  /* Its idle process, feed it! */
 	  feed_child(proc);
+	  proc->hungry = 1;	/* Simulate hunger.. */
+	  pick_next_vertex(proc,1);
+	  for (;;) {
+	    proc->hungry = 1;
+	    feed_child(proc);
+	    if (proc->fed)
+	      proc->overfed += 1;
+	    /* See if we should, and can feed more! */
+	    if (proc->thg == NULL ||
+		proc->pid == 0    ||
+		proc->thread == NULL)
+	      break;		/* No new active threads/vertices/proc.. */
+	    if (proc->cmdbuf[0] != 0)
+	      break;		/* it is already overstuffed */
+	    if (proc->overfed >= proc->thg->ce.overfeed)
+	      break;		/* if the limit is zero, don't overfeed ever.*/
+	    /* Ok, increment the counter, and loop back.. */
+	    proc->fed = 1;	/* tell the picker, it has been fed.. */
+	    pick_next_vertex(proc,1);
+	    if (proc->fed != 0) /* Out of jobs in this thread.. */
+	      break;
+	  }
+	  proc->hungry = 0; /* ... satiated.. */
 	  return 1;
 	}
 
@@ -688,25 +763,25 @@ struct thread *thr;
 
 	vp = thr->vertices;
 
-	if (numkids >= vp->ce.maxkids)
-	  vp->ce.pending = SIZE_L;
-	else if (vp->orig[L_CHANNEL]->kids >= vp->ce.maxkidChannel)
-	  vp->ce.pending = L_CHANNEL;
-	else if (thg->transporters >= vp->ce.maxkidThreads)
-	  vp->ce.pending = L_HOST;
+	if (numkids >= ce->maxkids)
+	  vp->ce_pending = SIZE_L;
+	else if (vp->orig[L_CHANNEL]->kids >= ce->maxkidChannel)
+	  vp->ce_pending = L_CHANNEL;
+	else if (thg->transporters >= ce->maxkidThreads)
+	  vp->ce_pending = L_HOST;
 	else
-	  vp->ce.pending = 0;
+	  vp->ce_pending = 0;
 
-	if (vp->ce.pending) {
+	if (vp->ce_pending) {
 	  if (verbose)
 	    printf("%s: (%d %dC %dT) >= (%d %dC %dT)\n",
-		   vp->ce.command,
+		   ce->command,
 		   numkids,
 		   vp->orig[L_CHANNEL]->kids,
 		   thg->transporters,
-		   vp->ce.maxkids,
-		   vp->ce.maxkidChannel,
-		   vp->ce.maxkidThreads);
+		   ce->maxkids,
+		   ce->maxkidChannel,
+		   ce->maxkidThreads);
 	  /*
 	   * Would go over limit.  Rescheduling for the next
 	   * (single) interval works ok in many situation.
@@ -722,10 +797,13 @@ struct thread *thr;
 
 	/* Now we are ready to start a new child to run our bits */
 	
-	/* Clean vertices 'proc'-pointers,
-	   Randomize the order of thread vertices */
+	/* Clean vertices 'proc'-pointers,  randomize the
+	   order of thread vertices  (or sort by spool file
+	   ctime, if in AGEORDER..) */
 	thread_vertex_shuffle(thr);
 
+	thr->attempts += 1;
+
 	rc = start_child(thr->vertices,
 			 thr->vertices->orig[L_CHANNEL],
 			 thr->vertices->orig[L_HOST]);
@@ -755,8 +833,8 @@ int ok;
 
 	thr  = proc->thread;
 
-if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx, jobs=%d ok=%d)\n",
-		    proc->tofd, (long)thr, thr ? thr->jobs : 0, ok);
+if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%p, jobs=%d ok=%d)\n",
+		    proc->tofd, thr, thr ? thr->jobs : 0, ok);
 
 	if (proc->pid < 0) {	/* "Jim, He is dead!"		*/
 	  if (proc->thread != NULL)
@@ -778,7 +856,6 @@ if (verbose) printf(" ... NONE, we are idle.\n");
 if (verbose) printf(" ... NONE, current one has not been fed..\n");
 	  return; /* Current one has not been fed..	*/
 	}
-	proc->fed = 0;
 
 	thr0 = thr;
 	thg  = thr->thgrp;
@@ -792,7 +869,8 @@ if (verbose) printf(" ... NONE, current one has not been fed..\n");
 	  /* Is the current one in processing ? */
 	  if (vtx->proc == NULL) {
 	    proc->vertex = vtx;
-if (verbose) printf(" ... thr=same vtx=0x%x\n",(long)vtx);
+if (verbose) printf(" ... thr=same vtx=0x%p\n",vtx);
+	    proc->fed = 0;
 	    return; /* No, it is eligible! */
 	  }
 
@@ -801,6 +879,14 @@ if (verbose) printf(" ... thr=same vtx=0x%x\n",(long)vtx);
 	}
 	/* Umm.. All vertices on this thread used! */
 
+	if (proc->overfed > 0) {
+	  /* We are/have overfed, DO NOT CHANGE THREAD YET */
+	  proc->fed = 1;
+if (verbose) printf(" ... overfed=%d, no thread change.\n",proc->overfed);
+	  return;
+	}
+
+
 	/* Pick another thread in our ring */
 	thr = thr->nextthg;
 
@@ -816,8 +902,6 @@ if (verbose) printf(" ... thr=same vtx=0x%x\n",(long)vtx);
 	  if (thr->wakeup > now && thr->attempts > 0)
 	    continue; /* wakeup in future, unless first time around! */
 
-	  thr->attempts += 1;
-
 	  /* Ok, this thread isn't busy, pick the first vertex */
 
 	  /* Get rid of the old host web */
@@ -825,10 +909,13 @@ if (verbose) printf(" ... thr=same vtx=0x%x\n",(long)vtx);
 	  if (proc->ho->kids == 0 && proc->ho->link == NULL)
 	    unweb(L_HOST,proc->ho);
 
-	  /* Clean vertices 'proc'-pointers,
-	     Randomize the order of thread vertices */
+	  /* Clean vertices 'proc'-pointers,  randomize the
+	     order of thread vertices  (or sort by spool file
+	     ctime, if in AGEORDER..) */
 	  thread_vertex_shuffle(thr);
 
+	  thr->attempts += 1;
+
 	  proc->vertex = thr->vertices;
 	  proc->thread = thr;
 	  proc->ch = thr->vertices->orig[L_CHANNEL];
@@ -842,9 +929,10 @@ if (verbose) printf(" ... thr=same vtx=0x%x\n",(long)vtx);
 				thread so that at the next time we
 				pick some other thread when skipping
 				failed threads.. */
-if (verbose) printf(" ... thr=0x%x vtx=0x%x ch=%s ho=%s\n",
-		    (long)thr,(long)proc->vertex,
+if (verbose) printf(" ... thr=0x%p vtx=0x%p ch=%s ho=%s\n",
+		    thr,proc->vertex,
 		    proc->ch->name,proc->ho->name);
+	  proc->fed = 0;
 	  return; /* It is eligible to run! */
 	}
 
@@ -889,14 +977,14 @@ time_t retrytime;
 	} else if (vtx->wakeup < now-7200 /* more than 2h in history .. */ )
 	  vtx->wakeup = now;
 
-	if (vtx->ce.nretries <= 0) {
+	if (vtx->thgrp->ce.nretries <= 0) {
 	  if (verbose)
-	    printf("...ce.retries = %d\n", vtx->ce.nretries);
+	    printf("...ce->retries = %d\n", vtx->thgrp->ce.nretries);
 	  return;
 	}
 
-	if (thr->retryindex >= vtx->ce.nretries)
-	  thr->retryindex = ranny(vtx->ce.nretries-1);
+	if (thr->retryindex >= vtx->thgrp->ce.nretries)
+	  thr->retryindex = ranny(vtx->thgrp->ce.nretries-1);
 	
 	/*
 	 * clamp retry time to a predictable interval so we
@@ -905,43 +993,59 @@ time_t retrytime;
 	if (retrytime < now+63)
 	  vtx->wakeup = now;
 #if 0
-	skew = vtx->wakeup % vtx->ce.interval;
-	if (skew <= vtx->ce.interval / 2)
-	  skew = - (skew + (vtx->ce.skew - 1));
+	skew = vtx->wakeup % vtx->thgrp->ce.interval;
+	if (skew <= vtx->thgrp->ce.interval / 2)
+	  skew = - (skew + (vtx->thgrp->ce.skew - 1));
 	else
-	  skew = skew + (vtx->ce.skew - 1);
-	skew = skew / vtx->ce.skew; /* want int div truncation */
+	  skew = skew + (vtx->thgrp->ce.skew - 1);
+	skew = skew / vtx->thgrp->ce.skew; /* want int div truncation */
 	
 	vtx->wakeup += (skew +
-			vtx->ce.retries[thr->retryindex] * vtx->ce.interval);
+			vtx->thgrp->ce.retries[thr->retryindex] * vtx->thgrp->ce.interval);
 #else
 	/* Actually we do NOT want to have synchronization of threads,
 	   as such causes simultaneous start of transporters, which
 	   causes "somewhat" spiky load behaviour */
 
-	vtx->wakeup += vtx->ce.retries[thr->retryindex] * vtx->ce.interval;
+	vtx->wakeup += vtx->thgrp->ce.retries[thr->retryindex] * vtx->thgrp->ce.interval;
 #endif
 	thr->retryindex++;
 
 	/* XX: change this to a mod expression */
 	if (vtx->wakeup < now)
-	  vtx->wakeup = ((((now - vtx->wakeup) / vtx->ce.interval)+1)
-			 * vtx->ce.interval);
+	  vtx->wakeup = ((((now - vtx->wakeup) / vtx->thgrp->ce.interval)+1)
+			 * vtx->thgrp->ce.interval);
 
 	wakeup = vtx->wakeup;
 
 	if (retrytime < now+63)
 	  retrytime = wakeup;
 
-	/* Time to expire ? */
-	if (vtx->ce.expiry > 0 && vtx->ce.expiry <= now && vtx->attempts > 0) {
-	  /* ... and now expire it! */
-	  expire(vtx,index);
-	  return;
-	}
-
 	/* Reschedule ALL vertices on this thread */
 	while (vtx) {
+
+	  /* Time to expire ? */
+	  if (vtx->ce_expiry > 0 && vtx->ce_expiry <= now &&
+	      vtx->attempts  > 0) {
+	    struct vertex *nvtx = vtx->nextitem;
+	    /* ... and now expire it! */
+	    /* this MAY invalidate also the THREAD object! */
+	    if (thr->jobs > 1) {
+if (nvtx == NULL)
+fprintf(stderr,"threads:reschedule() - expiring with thr->jobs=%d, nvtx == NULL!\n",
+	thr->jobs);
+	      expire(vtx,index);
+	    } else {
+if (nvtx != NULL)
+fprintf(stderr,"threads:reschedule() - expiring with thr->jobs=%d, nvtx != NULL!\n",
+	thr->jobs);
+	      expire(vtx,index);
+	      thr = NULL; /* The THR-pointed object is now invalid */
+	    }
+	    vtx = nvtx;
+	    continue;
+	  }
+
 	  if (vtx->wakeup < retrytime)
 	    vtx->wakeup = retrytime;
 	  if (wakeup > vtx->wakeup || wakeup == 0)
@@ -952,7 +1056,8 @@ time_t retrytime;
 
 	  vtx = vtx->nextitem;
 	}
-	thr->wakeup = wakeup;
+	if (thr != NULL)
+	  thr->wakeup = wakeup;
 }
 
 
@@ -972,6 +1077,8 @@ reschedule(vp, factor, index)
 	int skew;
 	struct vertex *ap = NULL, *pap = NULL;
 	struct thread *thr = vp->thread;
+	struct threadgroup *thg = vp->thgrp;
+	struct config_entry *ce = &(thg->ce);
 
 #if 0 /* Hmm.. The reschedule() is called only when we have a reason
 	 to call it, doesn't it ?  */
@@ -983,9 +1090,9 @@ reschedule(vp, factor, index)
 	time(&now);
 
 	if (verbose)
-	  printf("reschedule %lx now %d expiry in %d attempts %d factor %d inum %d (%s/%s: %s)\n",
-		 (u_long)vp, (int)now,
-		 (int)((vp->ce.expiry > 0) ? (vp->ce.expiry - now) : -999),
+	  printf("reschedule %p now %d expiry in %d attempts %d factor %d inum %d (%s/%s: %s)\n",
+		 vp, (int)now,
+		 (int)((vp->ce_expiry > 0) ? (vp->ce_expiry - now) : -999),
 		 vp->attempts,
 		 factor, (int)(vp->cfp->id),
 		 vp->orig[L_CHANNEL]->name,
@@ -999,54 +1106,57 @@ reschedule(vp, factor, index)
 	} else if (vp->wakeup < now-7200 /* more than 2h .. */ )
 	  vp->wakeup = now;
 
-	if (vp->ce.nretries <= 0) {
+	if (ce->nretries <= 0) {
 	  if (verbose)
-	    printf("ce.retries = %d\n", vp->ce.nretries);
+	    printf("ce->retries = %d\n", ce->nretries);
 	  return;
 	}
 #if 0
-	if (vp->ce.expiry > 0 && vp->ce.expiry <= now && vp->attempts > 0) {
+	if (vp->ce_expiry > 0 && vp->ce_expiry <= now && vp->attempts > 0) {
 	  if (verbose)
-	    printf("ce.expiry = %d, %d attempts\n",
-		   (int)(vp->ce.expiry), vp->attempts);
+	    printf("ce_expiry = %d, %d attempts\n",
+		   (int)(vp->ce_expiry), vp->attempts);
 	  expire(vp, index);
 	  return;
 	}
 #endif
-	if (factor == -1) {
-	  if (thr->retryindex >= vp->ce.nretries)
-	    thr->retryindex = ranny(vp->ce.nretries-1);
+	if (factor == -1 && vp->attempts) {
+	  if (thr->retryindex >= ce->nretries)
+	    thr->retryindex = ranny(ce->nretries-1);
 
 	  /*
 	   * clamp retry time to a predictable interval so we
 	   * eventually bunch up deliveries.
 	   */
-	  skew = vp->wakeup % vp->ce.interval;
-	  if (skew <= vp->ce.interval / 2)
-	    skew = - (skew + (vp->ce.skew - 1));
+	  skew = vp->wakeup % ce->interval;
+	  if (skew <= ce->interval / 2)
+	    skew = - (skew + (ce->skew - 1));
 	  else
-	    skew = skew + (vp->ce.skew - 1);
-	  skew = skew / vp->ce.skew; /* want int div truncation */
+	    skew = skew + (ce->skew - 1);
+	  skew = skew / ce->skew; /* want int div truncation */
 
 	  vp->wakeup += (skew +
-			 vp->ce.retries[thr->retryindex] * vp->ce.interval);
+			 ce->retries[thr->retryindex] * ce->interval);
 	  thr->retryindex++;
 	} else if (factor < -1) {
 	  vp->wakeup = -factor;
 	} else
-	  vp->wakeup += factor * vp->ce.interval;
+	  vp->wakeup += factor * ce->interval;
+
+	if (vp->attempts == 0)
+	  vp->wakeup = now;
 
 	/* XX: change this to a mod expression */
 	if (vp->wakeup < now)
-	  vp->wakeup = ((((now - vp->wakeup) / vp->ce.interval)+1)
-			* vp->ce.interval);
+	  vp->wakeup = ((((now - vp->wakeup) / ce->interval)+1)
+			* ce->interval);
 
-	if (vp->ce.expiry > 0
-	    && vp->ce.expiry <= vp->wakeup
+	if (vp->ce_expiry > 0
+	    && vp->ce_expiry <= vp->wakeup
 	    && vp->attempts > 0) {
 	  if (verbose)
-	    printf("ce.expiry = %d, %d attempts\n",
-		   (int)(vp->ce.expiry), vp->attempts);
+	    printf("ce_expiry = %d, %d attempts\n",
+		   (int)(vp->ce_expiry), vp->attempts);
 
 	  /* expire() will delete this vertex in due time */
 	  expire(vp, index);
@@ -1060,22 +1170,34 @@ reschedule(vp, factor, index)
 	   WE MUST LINK IT BACK!                      */
 
 	if (verbose)
-	  printf("wakeup %d pending %d\n", (int)(vp->wakeup), vp->ce.pending);
+	  printf("wakeup %d pending %d\n", (int)(vp->wakeup), vp->ce_pending);
 
 	/* link it back in at the right spot */
 	for (ap = thr->vertices, pap = NULL ; ap != NULL; ap = ap->nextitem) {
-	  if (ap->wakeup > vp->wakeup) {
+	  if (thr->thgrp->ce.flags & CFG_AGEORDER) {
+	    /* compare expiry times .. */
+	    if (ap->ce_expiry > vp->ce_expiry) {
+	      /* Link the VP in front of AP */
+	      _thread_linkfront(thr,ap,vp);
+	      thr->jobs += 1;
 
-	    /* Link the VP in front of AP */
-	    _thread_linkfront(thr,ap,vp);
-	    thr->jobs += 1;
+	      break;
+	    }
+	  } else {
+	    /* Compare wakeup timestamps.. */
+	    if (ap->wakeup > vp->wakeup) {
 
-	    break;
+	      /* Link the VP in front of AP */
+	      _thread_linkfront(thr,ap,vp);
+	      thr->jobs += 1;
+
+	      break;
+	    }
 	  }
 	  pap = ap;
 	}
 	if (verbose)
-	  printf("ap %lx pap %lx curitem %lx\n", (long)ap, (long)pap, (long)thr->vertices);
+	  printf("ap %p pap %p curitem %p\n", ap, pap, thr->vertices);
 
 	if (ap == NULL) {
 	  /* append to list */
@@ -1178,22 +1300,25 @@ FILE *fp;
 	struct threadgroup *thg;
 	int thg_once = 1;
 	int jobsum = 0, jobtotal = 0;
+	int threadsum = 0;
 	char timebuf[20];
 
 	extern time_t sched_starttime; /* From main() */
 
+	time(&now);
+
 	if (thrg_root == NULL) {
-	  fprintf(fp,"No threads/processes.\n");
+	  *timebuf = 0;
+	  saytime((long)(now - sched_starttime), timebuf, 1);
+	  fprintf(fp,"No threads/processes.  Uptime: %s\n",timebuf);
 	  return;
 	}
 
-	time(&now);
-
 	for (thg = thrg_root;
 	     thg_once || thg != thrg_root;
 	     thg = thg->next) {
 	  int cnt, procs;
-	  struct procinfo *p = thg->idleproc;
+	  struct procinfo *p;
 	  struct thread *thr;
 	  int thr_once = 1;
 
@@ -1217,24 +1342,66 @@ FILE *fp;
 		thr->proc->thread == thr) {
 	      ++procs;
 	      fprintf(fp," pid=%d",thr->proc->pid);
+	      fprintf(fp," hage=%ds",(now - thr->proc->hungertime));
+	      if (thr->proc->feedtime == 0)
+		fprintf(fp," fage=never");
+	      else
+		fprintf(fp," fage=%ds",(now - thr->proc->feedtime));
+	      fprintf(fp," ofed=%d", thr->proc->overfed);
 	    } else if (thr->wakeup > now)
 	      fprintf(fp," wakeup=%d",(int)(thr->wakeup - now));
-	    if (thr->vertices && thr->vertices->ce.pending)
+	    if (thr->vertices && thr->vertices->ce_pending)
 	      fprintf(fp,
-		      thr->vertices->ce.pending == SIZE_L ? "" :
-		      (thr->vertices->ce.pending == L_CHANNEL ? " channelwait" : " threadwait"));
+		      thr->vertices->ce_pending == SIZE_L ? "" :
+		      (thr->vertices->ce_pending == L_CHANNEL ? " channelwait" : " threadwait"));
 	    fprintf(fp,"\n");
 	  }
-	  fprintf(fp,"\t\tThreads: %d/%d ",thg->threads,cnt);
-	  fprintf(fp,"Procs: %d/%d ",thg->transporters,procs);
+	  fprintf(fp,"\t\tThreads: %d",thg->threads);
+	  if (thg->threads != cnt)
+	    fprintf(fp,"/%d",cnt);
+	  fprintf(fp," Procs: %d",thg->transporters);
+	  if (thg->transporters != procs)
+	    fprintf(fp,"/%d",procs);
+	  fprintf(fp," Plim: %d Flim: %d ",thg->ce.maxkidThreads,thg->ce.overfeed);
 	  fprintf(fp,"Rcpts: %d ",jobsum);
-	  jobtotal += jobsum;
+	  jobtotal  += jobsum;
+	  threadsum += thg->threads;
 	  cnt = 0;
-	  for (;p != 0; p = p->next) ++cnt;
-	  fprintf(fp,"Idle-procs: %d/%d\n",thg->idlecnt,cnt);
+	  for (p = thg->idleproc; p != 0; p = p->next) ++cnt;
+	  fprintf(fp,"Idle: %d",thg->idlecnt);
+	  if (thg->idlecnt != cnt)
+	    fprintf(fp,"/%d",cnt);
+	  fprintf(fp,"\n");
 	}
 	*timebuf = 0;
 	saytime((long)(now - sched_starttime), timebuf, 1);
-	fprintf(fp,"NumKids: %d   Idleprocs: %d   Messages: %d   Recipients: %d   Uptime: %s\n",
-		numkids, idleprocs, global_wrkcnt, jobtotal, timebuf);
+	fprintf(fp,"Kids: %d   Idle: %d   Msgs: %d   Thrds: %d   Rcpnts: %d   Uptime: %s\n",
+		numkids, idleprocs, global_wrkcnt, threadsum, jobtotal, timebuf);
+}
+
+int thread_count_recipients()
+{
+	struct threadgroup *thg;
+	struct thread *thr;
+	int thg_once = 1;
+	int jobsum, jobtotal = 0;
+
+	if (thrg_root == NULL)
+	  return 0;
+
+	for (thg = thrg_root;
+	     thg_once || thg != thrg_root;
+	     thg = thg->next) {
+	  int thr_once = 1;
+
+	  thg_once = 0;
+	  jobsum = 0;
+	  thr_once = 1;
+	  for (thr = thg->thread; thr && (thr_once || thr != thg->thread); thr = thr->nextthg) {
+	    thr_once = 0;
+	    jobsum += thr->jobs;
+	  }
+	  jobtotal += jobsum;
+	}
+	return jobtotal;
 }
