@@ -47,7 +47,6 @@ int	numkids = 0;
 
 extern char *getzenv();
 extern char *strchr();
-extern int cistrcmp();
 extern int errno;
 extern time_t time();
 
@@ -57,42 +56,82 @@ void
 feed_child(proc)
 struct procinfo *proc;
 {
-	char buf[500]; /* 60 or so SHOULD suffice.. */
+	struct vertex *vtx = proc->vertex;
 	int len, rc;
 
 	if (proc->thread == NULL) return; /* Might be called without next process.. */
 
-	if (proc->hungry) --hungry_childs;
-	/* It will be fed, clear this flag.. */
-	proc->hungry = 0;
+	if (proc->cmdbuf[0] != 0) {
+	  /* We have some leftovers from previous feed..
+	     .. feed them now.  */
+	  len = strlen(proc->cmdbuf);
+	  rc = write(proc->tofd, proc->cmdbuf, len);
+	  if (rc != len) {
+	    if (rc < 0 && (errno != EAGAIN && errno != EINTR &&
+			   errno != EWOULDBLOCK)) {
+	      /* Some real failure :-( */
+	      pipes_shutdown_child(proc->tofd);
+	      proc->tofd = -1;
+	      vtx->proc = NULL;
+	      vtx       = NULL;
+	      proc->thread->proc = NULL;
+	      proc->thread       = NULL;
+	      proc->cmdbuf[0] = 0;
+	      return;
+	    }
+	    if (rc < 0) rc = 0;
+	    if (rc > 0) {
+	      strcpy(proc->cmdbuf, proc->cmdbuf+rc);
+	    }
+	  } else {
+	    proc->cmdbuf[0] = 0; /* Clean the pending info.. */
+	  }
+	  proc->feedtime = now;
+	  return; /* We return latter.. */
+	}
 
-	proc->vertex->proc = proc;    /* Flag that it is in processing */
-	proc->vertex->ce.pending = 0; /* and clear the pending.. */
+	vtx->proc = proc;    /* Flag that it is in processing */
+	vtx->ce_pending = 0; /* and clear the pending.. */
 
 	if (proc->thg->withhost)
-	  sprintf(buf, "%s\n", proc->vertex->cfp->mid);
+	  sprintf(proc->cmdbuf, "%s\n", vtx->cfp->mid);
 	else
-	  sprintf(buf, "%s\t%s\n", proc->vertex->cfp->mid, proc->ho->name);
+	  sprintf(proc->cmdbuf, "%s\t%s\n", vtx->cfp->mid, proc->ho->name);
 
-	len = strlen(buf);
+	len = strlen(proc->cmdbuf);
 
 	if (verbose)
-	  printf("feed: tofd=%d, len=%d, chan=%s, buf=%s",
-		 proc->tofd,len,proc->ch->name,buf);
+	  printf("feed: tofd=%d, fed=%d, len=%d, chan=%s, buf=%s",
+		 proc->tofd,proc->fed,len,proc->ch->name,proc->cmdbuf);
 
-	rc = write(proc->tofd, buf, len);
+	rc = write(proc->tofd, proc->cmdbuf, len);
 
 	if (rc != len) {
-	  /* Failed ?  How come ?
-	     Close the 'tofd' pipe, and clear proc ptr. */
-	  pipes_shutdown_child(proc->tofd);
-	  proc->tofd = -1;
-	  proc->vertex->proc = NULL;
-	  proc->vertex       = NULL;
-	  proc->thread->proc = NULL;
-	  proc->thread       = NULL;
-	} else
-	  proc->fed = 1;
+	  if (rc < 0 && (errno != EAGAIN && errno != EINTR &&
+			 errno != EWOULDBLOCK)) {
+	    /* Failed ?  How come ?
+	       Close the 'tofd' pipe, and clear proc ptr. */
+	    pipes_shutdown_child(proc->tofd);
+	    proc->tofd = -1;
+	    vtx->proc = NULL;
+	    vtx       = NULL;
+	    proc->thread->proc = NULL;
+	    proc->thread       = NULL;
+	    proc->cmdbuf[0] = 0;
+	  } else {
+	    if (rc < 0) rc = 0; /* Interrupted write,
+				   we return to it latter.. */
+	    strcpy(proc->cmdbuf, proc->cmdbuf+rc);
+	  }
+	} else {
+	  if (proc->hungry) --hungry_childs;
+	  proc->cmdbuf[0] = 0; /* Nothing pending */
+	}
+	/* It was fed, clear this flag.. */
+	proc->hungry = 0;
+	proc->fed = 1;
+	proc->feedtime = now;
+	vtx->attempts += 1;
 }
 
 /*
@@ -112,7 +151,7 @@ start_child(vhead, chwp, howp)
 	  printf("transport(vhead,chan=%s,host=%s)\n",
 		 chwp->name,howp->name);
 
-	if (vhead->ce.argv == NULL) {
+	if (vhead->thgrp->ce.argv == NULL) {
 	  fprintf(stderr, "No command defined for %s/%s!\n",
 		  chwp->name, howp->name);
 	  return 0;
@@ -121,14 +160,14 @@ start_child(vhead, chwp, howp)
 	 * Replace the $host and $channel strings in the command line.
 	 */
 	os = buf;
-	for (i = 0; vhead->ce.argv[i] != NULL; ++i) {
-	  if (cistrcmp(vhead->ce.argv[i], replhost) == 0) {
+	for (i = 0; vhead->thgrp->ce.argv[i] != NULL; ++i) {
+	  if (strcmp(vhead->thgrp->ce.argv[i], replhost) == 0) {
 	    av[i] = howp->name;
-	  } else if (cistrcmp(vhead->ce.argv[i], replchannel) == 0) {
+	  } else if (strcmp(vhead->thgrp->ce.argv[i], replchannel) == 0) {
 	    av[i] = chwp->name;
-	  } else if (strchr(vhead->ce.argv[i], '$') != NULL) {
+	  } else if (strchr(vhead->thgrp->ce.argv[i], '$') != NULL) {
 	    s = os;
-	    for (cp = vhead->ce.argv[i]; *cp != '\0'; ++cp) {
+	    for (cp = vhead->thgrp->ce.argv[i]; *cp != '\0'; ++cp) {
 	      if (*cp == '$' && *(cp+1) == '{') {
 		cp += 2;
 		ocp = cp;
@@ -148,7 +187,7 @@ start_child(vhead, chwp, howp)
 	    av[i] = os;
 	    os = s + 1;
 	  } else
-	    av[i] = vhead->ce.argv[i];
+	    av[i] = vhead->thgrp->ce.argv[i];
 	}
 	av[i] = NULL;
 
@@ -172,9 +211,9 @@ static int runcommand(argv, vhead, chwp, howp)
 	static int pipesize = 0;
 
 
-	uid = vhead->ce.uid;
-	gid = vhead->ce.gid;
-	cmd = vhead->ce.command;
+	uid = vhead->thgrp->ce.uid;
+	gid = vhead->thgrp->ce.gid;
+	cmd = vhead->thgrp->ce.command;
 
 	if (pipes_create(to,from) < 0) return 0;
 	if (pipesize == 0)
@@ -243,9 +282,17 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	  memset(cpids, 0, sizeof(struct procinfo) * i);
 	}
 	proc = &cpids[fromfd];
+	memset(proc,0,sizeof(struct procinfo));
+#if 0 /* the memset() does this more efficiently.. */
 	proc->next   = NULL;
-	proc->pid    = pid;
+	proc->cmdbuf[0] = 0;
 	proc->reaped = 0;
+	proc->carryover = NULL;
+	proc->hungry = 0;
+	proc->fed    = 0;
+	proc->overfed = 0;
+#endif
+	proc->pid    = pid;
 	proc->ch     = chwp;
 	proc->ho     = howp;
 	proc->vertex = vhead;
@@ -258,22 +305,37 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	if (howp != NULL) howp->kids += 1;
 	proc->tofd   = tofd;
 	vhead->proc  = proc;
-	proc->carryover = NULL;
-	proc->hungry = 0;
-	proc->fed    = 0;
+	time(&proc->hungertime); /* Actually it is not yet 'hungry' as
+				    per reporting so, but we store the
+				    time-stamp anyway */
 #ifdef O_NONBLOCK /* POSIXy thing */
 	/* set up non-blocking I/O */
 	i = fcntl(fromfd, F_GETFL, 0);
 	i |= O_NONBLOCK;
 	i = fcntl(fromfd, F_SETFL, i);
+	if (fromfd != tofd) {
+	  i = fcntl(tofd, F_GETFL, 0);
+	  i |= O_NONBLOCK;
+	  i = fcntl(tofd, F_SETFL, i);
+	}
 #else
 #ifdef	USE_BSDNODELAY
+	i = fcntl(fromfd, F_GETFL, 0);
 #ifdef	FNONBLOCK
 	i |= FNONBLOCK;
 #else
 	i |= FNDELAY;
 #endif
 	i = fcntl(fromfd, F_SETFL, i);
+	if (fromfd != tofd) {
+	  i = fcntl(tofd, F_GETFL, 0);
+#ifdef	FNONBLOCK
+	  i |= FNONBLOCK;
+#else
+	  i |= FNDELAY;
+#endif
+	  i = fcntl(tofd, F_SETFL, i);
+	}
 #endif	/* USE_BSDNODELAY */
 #endif	/* O_NONBLOCK  */
 }
@@ -319,22 +381,18 @@ if (verbose)
 
 	/* ... but only if we were not in IDLE chain! */
 	if (proc->thread != NULL) {
-	  nvp = proc->thread->vertices;
+	  /* Reschedule them all .. */
+	  thread_reschedule(proc->thread,0,-1);
+	  /* Bookkeeping about dead transporters */
 	  proc->thg->transporters -= 1;
 	  --numkids;
 	  if (proc->vertex != NULL)
 	    proc->vertex->proc = NULL;
 	  proc->vertex = NULL;
-	  proc->thread->proc = NULL;
+	  if (proc->thread)
+	    proc->thread->proc = NULL;
 	  proc->thread = NULL;
 	  proc->thg    = NULL;
-	  /* XX: thread reschedule ?? */
-	  for (vp = nvp; vp != NULL ; vp = nvp) {
-	    nvp = vp->nextitem;
-	    if (vp->ngroup == 0)
-	      abort();
-	    reschedule(vp, 0, -1);
-	  }
 	} else {
 	  /* Maybe we were in idle chain! */
 	  if (proc->thg != NULL /* And not killed by  idle_cleanup() */) {
@@ -459,9 +517,11 @@ time_t timeout;
 {
 	int	i, n, rc, maxf;
 #ifdef _AIX
-	struct { fd_set fdsmask; fd_set msgsmask; } mask;
+	struct { fd_set fdsmask; fd_set msgsmask; } rdmask;
+	struct { fd_set fdsmask; fd_set msgsmask; } wrmask;
 #else
-	fd_set	mask;
+	fd_set	rdmask;
+	fd_set	wrmask;
 #endif
 	struct timeval tv;
 
@@ -476,15 +536,18 @@ time_t timeout;
 	tv.tv_usec = 0;
 
 	maxf = -1;
-	_FD_ZERO(mask);
+	_FD_ZERO(rdmask);
+	_FD_ZERO(wrmask);
 	if (cpids != NULL)
 	  for (i = 0; i < scheduler_nofiles ; ++i)
 	    if (cpids[i].pid != 0) {
-	      _FD_SET(i, mask);
+	      _FD_SET(i, rdmask);
 	      maxf = i;
+	      if (cpids[i].cmdbuf[0] != 0)
+		_FD_SET(i, wrmask);
 	    }
 	if (querysocket >= 0) {
-	  _FD_SET(querysocket, mask);
+	  _FD_SET(querysocket, rdmask);
 	  if (maxf < querysocket)
 	    maxf = querysocket;
 	}
@@ -493,7 +556,7 @@ time_t timeout;
 	++maxf;
 	/*fprintf(stderr, "about to select on %x [%d]\n",
 			  mask.fds_bits[0], maxf); */
-	if ((n = select(maxf, &mask, NULL, NULL, &tv)) < 0) {
+	if ((n = select(maxf, &rdmask, &wrmask, NULL, &tv)) < 0) {
 	  int err = errno;
 	  /* fprintf(stderr, "got an interrupt (%d)\n", errno); */
 	  in_select = 0;
@@ -501,10 +564,14 @@ time_t timeout;
 	    return 0;
 	  if (errno == EINVAL || errno == EBADF) {
 	    fprintf(stderr, "** select() returned errno=%d\n", err);
-	    for (i = 0; i < maxf; ++i)
-	      if (_FD_ISSET(i,mask) &&
+	    for (i = 0; i < maxf; ++i) {
+	      if (_FD_ISSET(i,rdmask) &&
 		  fcntl(i,F_GETFL,0) < 0)
-		fprintf(stderr,"** Invalid fd on a select() mask: %d\n",i);
+		fprintf(stderr,"** Invalid fd on a select() rdmask: %d\n",i);
+	      if (_FD_ISSET(i,wrmask) &&
+		  fcntl(i,F_GETFL,0) < 0)
+		fprintf(stderr,"** Invalid fd on a select() wrmask: %d\n",i);
+	    }
 	    fflush(stderr);
 	    abort();
 	  }
@@ -517,13 +584,13 @@ time_t timeout;
 	  in_select = 0;
 	  return 1;
 	} else {
-	  /*fprintf(stderr, "got %d ready (%x)\n", n, mask.fds_bits[0]);*/
-	  if (querysocket >= 0 && _FD_ISSET(querysocket, mask)) {
+	  /*fprintf(stderr, "got %d ready (%x)\n", n, rdmask.fds_bits[0]);*/
+	  if (querysocket >= 0 && _FD_ISSET(querysocket, rdmask)) {
 	    struct sockaddr_in raddr;
 	    int	raddrlen;
 
 	    --n;
-	    _FD_CLR(querysocket, mask);
+	    _FD_CLR(querysocket, rdmask);
 	    raddrlen = sizeof raddr;
 	    i = accept(querysocket, (struct sockaddr *)&raddr, &raddrlen);
 	    if (i < 0) {
@@ -541,16 +608,28 @@ time_t timeout;
 	      close(i);
 	    }
 	  }
-	  if (cpids != NULL)
+	  if (cpids != NULL) {
 	    for (i = 0; i < maxf; ++i) {
-	      if (cpids[i].pid != 0 && _FD_ISSET(i, mask)) {
+	      if (cpids[i].pid != 0 && _FD_ISSET(i, rdmask)) {
 		--n;
-		_FD_CLR(i, mask);
+		_FD_CLR(i, rdmask);
 		/*fprintf(stderr,"that is fd %d\n",i);*/
 		/* do non-blocking reads from this fd */
 		readfrom(i);
+		/* Because this loop might take a while ... */
+		queryipccheck();
 	      }
 	    }
+	    /* In case we have non-completed 'feeds', try feeding them */
+	    for (i = 0; i < scheduler_nofiles; ++i)
+	      if (cpids[i].pid > 0
+		  && cpids[i].cmdbuf[0] != 0
+		  && _FD_ISSET(i, wrmask)) {
+		feed_child(&cpids[i]);
+		/* Because this loop might take a while ... */
+		queryipccheck();
+	      }
+	  }
 	}
 	/* fprintf(stderr, "return from mux\n"); */
 	in_select = 0;
@@ -560,19 +639,19 @@ time_t timeout;
 void
 queryipccheck()
 {
-	int	n;
-#ifdef _AIX
-	struct { fd_set fdsmask; fd_set msgsmask; } mask;
-#else
-	fd_set	mask;
-#endif
-	struct timeval tv;
-
-	tv.tv_sec = 0;
-	tv.tv_usec = 0;
-
-	_FD_ZERO(mask);
 	if (querysocket >= 0) {
+	  int	n;
+#ifdef _AIX
+	  struct { fd_set fdsmask; fd_set msgsmask; } mask;
+#else
+	  fd_set	mask;
+#endif
+	  struct timeval tv;
+
+	  tv.tv_sec = 0;
+	  tv.tv_usec = 0;
+
+	  _FD_ZERO(mask);
 	  _FD_SET(querysocket, mask);
 
 	  n = select(querysocket+1, &mask, NULL, NULL, &tv);
@@ -872,7 +951,7 @@ int signum;
 	  pid = wait(&statloc);
 #endif
 #endif
-	  if (pid <= 0) return;
+	  if (pid <= 0) break;
 
 #ifdef	WIFEXITED
 	  if (WIFEXITED(statloc)) ok = 1;
