@@ -27,12 +27,12 @@ static void expaux __((struct vertex *, int, char *));
 
 extern char *strchr();
 extern char *strrchr();
-extern int cistrcmp();
 extern int ermdir();
 extern int rmdir();
 extern int eunlink();
 extern struct spblk *sp_fhead(), *sp_fnext();
 extern time_t time();
+extern time_t now;
 extern int global_wrkcnt;
 
 /*
@@ -80,37 +80,71 @@ update(fd, diagnostic)
 	struct procinfo *proc = &cpids[fd];
 
 	if (*diagnostic == 0) {
-	  /* Lone newline.. */
+	  /* Lone newline.. old-style indications from the transporter */
 	  if (proc->tofd >= 0 &&
 	      proc->hungry == 0) {
 	    proc->hungry = 1;
-	    time(&proc->hungertime);
+	    time(&now);
+	    proc->hungertime = now;
 	    ++hungry_childs;
 	  }
 	  /* Things are known to DIE from under us! */
-	  /* Pick next, and feed it! */
-	  pick_next_vertex(proc, 0);
+	  /* .. propably not a good idea to try to pick any next.. */
+	  if (proc->overfed > 0)
+	    proc->overfed -= 1;
+	  pick_next_vertex(proc, 1);
 	  if (proc->hungry)
 	    feed_child(proc);
 	  return;
 	}
 	if (*diagnostic == '#' /* DEBUG comment.. */) {
 	  if (strncmp(diagnostic,"#hungry",7)==0) {
-	    if (proc->tofd >= 0 &&
-		proc->hungry == 0) {
+	    if (proc->tofd   >= 0) {
 	      proc->hungry = 1;
 	      time(&proc->hungertime);
-	      ++hungry_childs;
-	    }
-	    /* Pick next, and feed it! */
-	    pick_next_vertex(proc,1);
-	    if (proc->hungry);
-	      feed_child(proc);
+	      if (proc->overfed > 0)
+		/* It was overfed, decrement that counter first.. */
+		proc->overfed -= 1;
+	      if (!proc->overfed) {
+		++hungry_childs;
+		/* Unless it is still overfed,
+		   Pick next, and feed it! */
+		pick_next_vertex(proc,1);
+		for (;;) {
+		  if (proc->hungry)
+		    feed_child(proc);
+		  if (proc->fed)
+		    proc->overfed += 1;
+		  /* See if we should, and can feed more! */
+		  if (proc->thg == NULL ||
+		      proc->pid == 0    ||
+		      proc->thread == NULL)
+		    break;	/* No new active threads/vertices/proc.. */
+		  if (proc->cmdbuf[0] != 0)
+		    break;	/* it is already overstuffed */
+		  if (proc->overfed >= proc->thg->ce.overfeed)
+		    break;	/* if the limit is zero, don't overfeed ever.*/
+		  /* Ok, increment the counter, and loop back.. */
+		  proc->hungry = 1; /* Simulate hunger.. */
+		  proc->fed = 1;    /* tell them, it has been fed.. */
+		  pick_next_vertex(proc,1);
+		  if (proc->fed != 0) /* Out of jobs in this thread.. */
+		    break;
+		}
+		proc->hungry = 0; /* ... satiated.. */
+	      } else {
+		if (verbose)
+		  printf("... child pid %d overfed=%d\n",
+			 proc->pid,proc->overfed);
+	      }
+	    } else
+	      if (verbose)
+		printf("'#hungry' from child without forward-channel\n");
 	    return;
-	  }
+	  } /* end of '#hungry' processing */
 	  fprintf(stderr, "%s DBGdiag: %s\n", timestring(), diagnostic);
 	  return;
-	}
+	} /* end of debug-diagnostics processing */
 	inum = atol(diagnostic);
 	if ((cp = strchr(diagnostic, '/')) == NULL) {
 	  fprintf(stderr, "%s Misformed diagnostic: %s\n",
@@ -144,7 +178,6 @@ update(fd, diagnostic)
 
 	if ((vp = findvertex(inum, offset, &index)) == NULL)
 	  return;
-	vp->attempts += 1;
 
 	if (vp->notary != NULL)
 	  free(vp->notary);
@@ -155,7 +188,9 @@ update(fd, diagnostic)
 	/* Select function by the type name: ok/error/retryat/deferred */
 
 	for (dcp = &diags[0]; dcp->name != NULL; ++dcp) {
-	  if (cistrcmp(dcp->name, type) == 0) {
+	  /* XX: replace strcmp() with cistrcmp() ??  Should not need,
+	     unless something is wrong with the transporters. */
+	  if (strcmp(dcp->name, type) == 0) {
 	    (dcp->fcn)(vp, index, inum, offset, notary, message);
 	    break;
 	  }
@@ -165,7 +200,6 @@ update(fd, diagnostic)
 		  timestring(), type);
 }
 
-
 /*
  * Deallocate a control file descriptor.
  */
@@ -186,7 +220,7 @@ unctlfile(cfp, no_unlink)
 	  eunlink(cfp->mid);
 	  --global_wrkcnt;
 	  if (verbose)
-	    printf("%s: unlink %s", cfp->logident, cfp->mid);
+	    printf("%s: unlink %s (mid=0x%p)", cfp->logident, cfp->mid, cfp->mid);
 
 	  sprintf(path, "../%s/%s", QUEUEDIR, cfp->mid);
 	  eunlink(path);
@@ -203,9 +237,7 @@ unctlfile(cfp, no_unlink)
 	spl = sp_lookup(cfp->id, spt_mesh[L_CTLFILE]);
 	if (spl != NULL)
 	  sp_delete(spl, spt_mesh[L_CTLFILE]);
-	if (cfp->contents != NULL)
-	  free(cfp->contents);
-	free((char *)cfp);
+	free_cfp_memory(cfp);
 }
 
 void unvertex(vp, justfree, ok)
@@ -217,9 +249,11 @@ void unvertex(vp, justfree, ok)
 	if (vp->ngroup > 0)
 	  return;
 
-	if (vp->proc && vp->proc->vertex == vp)
+	if (vp->proc && vp->proc->vertex == vp) {
+	  vp->proc->fed = 1; /* Mark it fed just in case.. */
 	  /* Pick next, but don't feed it (yet)! */
 	  pick_next_vertex(vp->proc, ok);
+	}
 
 	for (i = 0; i < SIZE_L; ++i) {
 	  if (vp->next[i] != NULL)
@@ -248,10 +282,9 @@ void unvertex(vp, justfree, ok)
 
 	web_disentangle(vp, ok); /* does also unthread() */
 
-	if (vp->message != NULL)
-	  free(vp->message);
-	if (vp->notary != NULL)
-	  free(vp->notary);
+	if (vp->message != NULL) free(vp->message);
+	if (vp->notary  != NULL) free(vp->notary);
+	/* if (vp->sender != NULL) free(vp->sender); */ /* XX: cache !! ?? */
 	free((char *)vp);
 }
 
@@ -443,7 +476,6 @@ saytime(value, buf, shortform)
 
 /*
  * vtxupdate() -- delete the vertex matching our (sub)index
- *
  */
 
 static void vtxupdate(vp, index, ok)
@@ -454,9 +486,12 @@ static void vtxupdate(vp, index, ok)
 
 	for (i = 0; i < vp->ngroup; ++i)
 	  if (vp->index[i] == index) {
-	    for (++i; i < vp->ngroup; ++i)
-	      vp->index[i-1] = vp->index[i];
+	    /* remove us from the vertex indices */
 	    vp->ngroup -= 1;
+	    /* compact the index array */
+	    for (++i; i <= vp->ngroup; ++i)
+	      vp->index[i-1] = vp->index[i];
+	    /* if none are left, unvertex() it.. */
 	    if (vp->ngroup <= 0)
 	      unvertex(vp, 0, ok);
 	    break;
@@ -488,8 +523,10 @@ static void expaux(vp, index, buf)
 
 	/* Report expiry */
 	for (i = 0 ; i < vp->ngroup; ++i)
-	  if (vp->index[i] == index)
+	  if (vp->index[i] == index) {
 	    msgerror(vp, vp->cfp->offset[index], buf);
+	    break;
+	  }
 
 	/* Log something into the scheduler log */
 	fprintf(stderr, "%s %s: %s/%s from %s %s\n", timestring(), progname,
@@ -514,18 +551,18 @@ expire(vp, index)
 
 	if (vp->notary == NULL) {
 	  /* addres / action / status / diagnostic / wtt */
-	  sprintf(buf,"%s\001%s\001%s\001%s\001",
-		  "**ADDR**", /* XX: recipient address! */
+	  sprintf(buf,"%s\001%s\001%s\001%s",
+		  "\003", /* XX: recipient address! XX: MAGIC INFO! */
 		  "failure",
 		  "4.0.0 (unspecified timeout failure)",
-		  "smtp; 400 (Expired after");
-	  saytime((u_long)(vp->ce.expiry - vp->cfp->ctime), buf, 0);
-	  strcat(buf,")");
+		  "smtp; 400 (Expired after ");
+	  saytime((u_long)(vp->ce_expiry - vp->cfp->ctime), buf, 0);
+	  strcat(buf,")\001");
 	  vp->notary = strsave(buf);
 	}
 
 	strcpy(buf, "expired after ");
-	saytime((u_long)(vp->ce.expiry - vp->cfp->ctime), buf, 0);
+	saytime((u_long)(vp->ce_expiry - vp->cfp->ctime), buf, 0);
 
 	if (vp->message != NULL && *(vp->message) != '\0') {
 	  emsg = emalloc(strlen(buf) + strlen(vp->message) + strlen(fmt));
@@ -534,7 +571,10 @@ expire(vp, index)
 	  emsg = buf;
 
 	if (index < 0) {
-	  for (i = 0 ; i < vp->ngroup; ++i)
+	  /* Expire from the LAST index to the first, this way
+	     we won't do mistake of calling indixes after they
+	     have been deleted.. */
+	  for (i = vp->ngroup -1; i >= 0; --i)
 	    expaux(vp, vp->index[i], emsg);
 	} else
 	  expaux(vp, index, emsg);
@@ -611,7 +651,7 @@ static int u_error(vp, index, inum, offset, notary, message)
 	  message = "(unknown)";
 	fprintf(stderr,
 		"%s %s: %ld/%ld/%s/error %s\n", timestring(),
-		vp->ce.command, inum, offset, notary, message);
+		vp->thgrp->ce.command, inum, offset, notary, message);
 
 	msgerror(vp, offset, message);
 
@@ -677,11 +717,13 @@ static int u_retryat(vp, index, inum, offset, notary, message)
 	 * the heuristic in reschedule() to ignore the request if
 	 * the time is already in the future should help out.
 	 */
-	thread_reschedule(vp->thread, retrytime, index);
 
+	/* ``vp'' might become expired by  thread_reschedule() .. */
 	if (vp->proc && vp->proc->vertex == vp)
 	  /* Pick next, but don't feed it (yet)! */
 	  pick_next_vertex(vp->proc, 0);
 
+	thread_reschedule(vp->thread, retrytime, index);
+
 	return 1;
 }
