@@ -531,9 +531,9 @@ main(argc, argv)
 	    for (msgfd = getdtablesize() - 1; msgfd >= 0; --msgfd)
 	      if (msgfd != s) {
 		/* junk any buffered output... */
-		if (msgfd == fileno(stdout))
+		if (msgfd == FILENO(stdout))
 		  fflush(stdout);
-		else if (msgfd == fileno(stderr))
+		else if (msgfd == FILENO(stderr))
 		  fflush(stderr);
 		close(msgfd);
 	      }
@@ -868,7 +868,7 @@ int insecure;
 	  return;
 	}
 
-	availspace = fd_statfs(fileno(mfp));
+	availspace = fd_statfs(FILENO(mfp));
 	if (availspace < 0)
 	  availspace = 2000000000; /* Over 2G ? */
 	availspace >>= 1;
@@ -1262,7 +1262,7 @@ smtpserver(insecure)
 	  *(cp + strlen(cp) - 1) = '\0';
 
 	localsocksize = sizeof(localsock);
-	if (getsockname(fileno(stdin),
+	if (getsockname(FILENO(stdin),
 			(struct sockaddr *)&localsock,&localsocksize) == 0 &&
 	    localsocksize == sizeof(localsock)) {
 	  /* Success with it :) */
@@ -1439,7 +1439,7 @@ smtpserver(insecure)
 		filsiz = mvdata(mfp, msg);
 		alarm(0);		/* cancel the alarm() */
 		tell = ftell(mfp);
-		if (fstat(fileno(mfp), &stbuf) < 0)
+		if (fstat(FILENO(mfp), &stbuf) < 0)
 		  inum = 0;
 		else
 		  inum = stbuf.st_ino;
