@@ -105,6 +105,9 @@ main(argc, argv)
 	  fflush(stdout);
 
 	  if (fgets(msgfilename, sizeof msgfilename, stdin) == NULL) break;
+	  if (strchr(msgfilename, '\n') == NULL) break; /* No ending '\n' !
+							   Must have been
+							   partial input! */
 
 	  /* Input:
 	       spool/file/name [ \t host.info ] \n
