@@ -198,6 +198,8 @@ main(argc, argv)
 	  fflush(stdout);
 	  if (fgets(file, sizeof file, stdin) == NULL)
 	    break;
+	  if (strchr(file, '\n') == NULL) break; /* No ending '\n' !  Must
+						    have been partial input! */
 	  if (emptyline(file, sizeof file))
 	    break;
 
@@ -446,6 +448,9 @@ struct qtypes {
 	{	"cname",	T_CNAME		},
 	{	"mx",		T_MX		},
 	{	"a",		T_A		},
+#ifdef  T_ANY
+	{	"any",		T_ANY		},
+#endif
 #ifdef	T_MP
 	{	"mp",		T_MP		},
 #endif	/* T_MP */
@@ -467,6 +472,8 @@ hold_ns(s)
 	struct qtypes *qtp;
 	char *cp, host[BUFSIZ];
 
+	res_init();
+
 	if ((cp = strrchr(s, '/')) == NULL)
 	  return 1;	/* human error, lets be nice */
 	if (cp > s && *(cp-1) == '.')
