@@ -278,6 +278,9 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	d.ctlid = atol(file);
 	d.senders = NULL;
 	d.recipients = NULL;
+	d.rcpnts_total = 0;
+	d.rcpnts_remaining = 0;
+	d.rcpnts_failed = 0;
 	d.logident = "none";
 	d.envid   = NULL;
 	d.verbose = NULL;
@@ -300,7 +303,13 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	    d.senders = ap;
 	    break;
 	  case _CF_RECIPIENT:
-	    if (*++s != _CFTAG_NORMAL || d.senders == NULL)
+	    ++s;
+	    /* Calculate statistics .. Scheduler asks for it.. */
+	    d.rcpnts_total += 1;
+	    if (*s == _CFTAG_NOTOK)   d.rcpnts_failed    += 1;
+	    else if (*s != _CFTAG_OK) d.rcpnts_remaining += 1;
+
+	    if (*s != _CFTAG_NORMAL || d.senders == NULL)
 	      break;
 	    ++s;
 	    if (*s >= 'a' && *s <= 'z') {
