@@ -1,13 +1,14 @@
 /*
  *	MIME-part-2 header 8-bit coding to MIME-coded-tokens
  *
- *      Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
+ *      Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1996
  */
 
 #include <varargs.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include "ta.h"
+#include "sysprotos.h"
 
 extern FILE *verboselog;
 
@@ -20,10 +21,16 @@ extern FILE *verboselog;
 #endif
 
 
+extern void *emalloc();
+extern void *realloc();
+
 int cvtspace_copy __((struct rcpt *rp));
 
+#undef USE_NEW_MIME2_CODE /* not yet ready.. */
+#ifdef USE_NEW_MIME2_CODE
+
 /*
- * The  header8bit2QP() -code is by  Markku T. Jarvinen <mta@funet.fi>
+ * The  header8bit2QP() -code is by  Markku T. Jarvinen <mta@sci.fi>
  * who made it for sendmail 8.7 (which lacks MIME-part-2)
  */
 
@@ -42,12 +49,12 @@ const char *Base16Code = "0123456789ABCDEF";
  * line will be split on whitespace if possible
  */
 
-int
-header8bit2QP(line, defcharset, out, size, fold)
+static int
+header8bit2QP(line, defcharset, outp, sizep, fold)
      unsigned char *line;
      const char *defcharset;
-     unsigned char *out;
-     int size;
+     unsigned char **outp;
+     int *sizep;
      int fold;
 {
 	unsigned char *ptr, *outptr, *outline;
@@ -57,9 +64,6 @@ header8bit2QP(line, defcharset, out, size, fold)
 	int fffppp;
   
 	linelen = strlen(line);
-	memset(out, 0, size);
-	if (linelen >= size)
-	  return 0;
 	ptr = line;
 	if (verboselog)
 	  fprintf(verboselog,"header8bit2QP: QP\n");
@@ -80,9 +84,11 @@ header8bit2QP(line, defcharset, out, size, fold)
 	    lines++;
 	  }
 	  abslen += strlen(word);
-	  if (abslen >= size)
-	    return 0;
-	  strcat(out, word);
+	  if (abslen >= *sizep) {
+	    *outp = realloc(*outp,abslen+2);
+	    *sizep = abslen+2;
+	  }
+	  strcat(*outp, word);
 	}
 #if 0
 	{
@@ -176,7 +182,7 @@ header8bit2QPnextword(line, word, len, inside, start_encode, fold, hdr_offset)
 	*tmp = '\0';
 	return 0;
 }	
-
+#endif
 
 /* This is rather half-baked approach, it creates correct tokens,
    but if two tokens wind up adjacent, or one's length exceeds
@@ -187,6 +193,7 @@ headers_to_mime2(rp,verboselog)
 	struct rcpt *rp;
 	FILE *verboselog;
 {
+#ifndef USE_NEW_MIME2_CODE
 	char **inhdr;
 
 	if (*(rp->newmsgheadercvt) == NULL)
@@ -250,5 +257,40 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	  ++inhdr;
 	}
 	return 0;
+#else /* USE_NEW_MIME2_CODE */
+	/* New MIME-2 code by mta@sci.fi in use.. */
+
+	char **inhdr, **inhdr1;
+	char *sumstr = emalloc(1);
+	int   sumlen = 0;
+
+	*sumstr = 0;
+
+	if (*(rp->newmsgheadercvt) == NULL)
+	  if (!cvtspace_copy(rp))
+	    return -1;	/* Failed to copy ! */
+
+	inhdr = *(rp->newmsgheadercvt);
+	inhdr1 = inhdr;
+
+	while (inhdr && *inhdr) {
+	  char *nn;
+	  int thislen = strlen(*inhdr);
+
+	  if (**inhdr == ' ' || **inhdr == '\t') {
+	    /* Continuation line.. */
+	    nn = realloc(sumstr,sumlen+thislen+2);
+	    if (!nn) return -1; /* AUTCH! */
+	    strcat(nn+sumlen,*inhdr);
+	    strcat(nn+sumlen+thislen,"\n");
+	    sumlen += thislen+1;
+	    sumstr = nn;
+	  } else {
+	    if (*sumstr != 0) {
+
+	    }
+	  }
+	}
+#endif /* USE_NEW_MIME2_CODE */
 }
 
