@@ -229,6 +229,7 @@ char *mime_boundary = NULL;
 int   mime_boundary_len = 0;
 int mmdf_mode = 0;		/* Write out MMDF-style mail folder
 				   ("\001\001\001\001" as the separator..)  */
+long eofindex = -1;		/* When negative, putmail() can't truncate() */
 
 extern SIGNAL_TYPE wantout();
 extern time_t time();
@@ -401,6 +402,8 @@ main(argc, argv)
 
 	  if (fgets(file, sizeof file, stdin) == NULL)
 	    break;
+	  if (strchr(file, '\n') == NULL) break; /* No ending '\n' !  Must
+						    have been partial input! */
 	  if (emptyline(file, sizeof file))
 	    break;
 
@@ -1002,9 +1005,10 @@ deliver(dp, rp, timestring)
 	alarm(0);
 
 
+	eofindex = lseek(fdmail, 0L, L_XTND);
 #if	defined(BIFF) || defined(RBIFF)
 	if (nbp != NULL)
-	  nbp->offset = lseek(fdmail, 0L, L_XTND);
+	  nbp->offset = eofindex;
 #endif	/* BIFF || RBIFF */
 	fp = putmail(dp, rp, fdmail, "a+", timestring, file);
 	
@@ -1055,7 +1059,6 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 {
 	FILE *fp;
 	int len, rc;
-	long eofindex;
 	char buf[2];
 	struct stat st;
 	char *fromuser;
@@ -1069,7 +1072,6 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	}
 
 	/* append From_ line and print out the header */
-	eofindex = ftell(fp);
 
 	if (!mmdf_mode &&
 	    *(rp->addr->user) != TO_PIPE && eofindex > 0L &&
@@ -1086,7 +1088,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	   * The mtime will be set to now by the following write() calls.
 	   */
 
-	  fstat(fileno(fp), &st);
+	  fstat(FILENO(fp), &st);
 	  if (!keepatime) st.st_atime = 0;
 
 	  len = fread(buf, 1, 2, fp);
@@ -1119,7 +1121,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	      /* XX: should I really do this? */
 #ifdef	USE_FTRUNCATE
 	      fflush(fp);
-	      ftruncate(fileno(fp), (u_long)eofindex);
+	      if (eofindex >= 0)
+		ftruncate(FILENO(fp), (u_long)eofindex);
 #endif /* USE_FTRUNCATE */
 	      fclose(fp);
 	      return NULL;
@@ -1153,7 +1156,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 		     "header write to \"%s\" failed", file);
 	  /* XX: should I really do this? */
 	  fflush(fp);
-	  ftruncate(fileno(fp), (u_long)eofindex);
+	  if (eofindex >= 0)
+	    ftruncate(FILENO(fp), (u_long)eofindex);
 	  fclose(fp);
 	  return NULL;
 	}
@@ -1162,7 +1166,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	if (lastch == EOF) {
 	  /* XX: should I really do this? */
 	  fflush(fp);
-	  ftruncate(fileno(fp), (u_long)eofindex);
+	  if (eofindex >= 0)
+	    ftruncate(FILENO(fp), (u_long)eofindex);
 	  fclose(fp);
 	  return NULL;
 	}
@@ -1183,7 +1188,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 		      file, strerror(errno));
 	  /* XX: should I really do this? */
 	  fflush(fp);
-	  ftruncate(fileno(fp), (u_long)eofindex);
+	  if (eofindex >= 0)
+	    ftruncate(FILENO(fp), (u_long)eofindex);
 	  fclose(fp);
 	  return NULL;
 	}
@@ -1328,6 +1334,7 @@ program(dp, rp, timestring)
 	errfp = fdopen(in[0], "r");
 	/* write the message */
 	mmdf_mode += 2;
+	eofindex = -1; /* NOT truncatable! */
 	fp = putmail(dp, rp, out[1], "w", timestring, rp->addr->user+1);
 	mmdf_mode -= 2;
 	if (fp == NULL) {
@@ -1629,16 +1636,16 @@ appendlet(dp, rp, fp, file, ismime)
 	  /* can we use cache of message body data ? */
 	  /* Split it to lines.. */
 	  if (readalready > 0) {
-	    int linelen;
+	    int linelen, readidx = 0;
 	    char *s0 = buffer, *s;
-	    while (readalready > 0) {
+	    while (readidx < readalready) {
 	      s = s0;
 	      linelen = 0;
-	      while (*s != 0 && *s != '\n' && readalready > 0) {
-		++s; ++linelen; --readalready;
+	      while (*s != 0 && *s != '\n' && readidx < readalready) {
+		++s; ++linelen; ++readidx;
 	      }
-	      if (*s == '\n' && readalready > 0) {
-		++s; ++linelen; --readalready;
+	      if (*s == '\n' && readidx < readalready) {
+		++s; ++linelen; ++readidx;
 	      }
 	      if (writemimeline(fp, s0, linelen, &lastch) != linelen) {
 		DIAGNOSTIC(rp, EX_IOERR,
@@ -1685,7 +1692,7 @@ appendlet(dp, rp, fp, file, ismime)
 
 	} else {		/* is NOT MIME message.. */
 
-	  /* can we use cache of message body data */
+	  /* can we use cache of message body data ? */
 	  if (readalready != 0) {
 	    if (writebuf(fp, buffer, readalready, &lastch) != readalready) {
 	      DIAGNOSTIC(rp, EX_IOERR,
