@@ -207,6 +207,8 @@ main(argc, argv)
 
 	  if (fgets(file, sizeof file, stdin) == NULL)
 	    break;
+	  if (strchr(file, '\n') == NULL) break; /* No ending '\n' !  Must
+						    have been partial input! */
 	  if (emptyline(file, sizeof file))
 	    break;
 
