@@ -202,7 +202,7 @@ int first_uid = 0;		/* Make the opening connect with the UID of the
 
 int D_alloc = 0;		/* Memory usage debug */
 
-static char FAILED[] = "failed";
+static const char FAILED[] = "failed";
 
 #ifdef	lint
 #undef	putc
@@ -226,6 +226,7 @@ long ehlo_sizeval      = 0;
 int  esmtp_on_banner = -1;
 int  main_esmtp_on_banner = -1;
 
+time_t now;
 
 extern int errno;
 extern void warning(), report();
@@ -259,7 +260,7 @@ extern void notarystatsave __((char *smtpstatline, char *status));
 
 extern void prversion();
 extern void getnobody();
-extern int makeconn __((struct hostent*, int*, FILE*, FILE**,int));
+extern int makeconn __((struct hostent*, int*, FILE*, FILE**));
 extern int vcsetup __((char*,int,int*,char*,FILE*));
 extern void hp_init __((struct hostent*));
 extern char **hp_getaddr __((void)), **hp_nextaddr __((void));
@@ -287,6 +288,30 @@ extern void getdaemon();
 FILE *verboselog = NULL;
 int smtp_isopen = 0;
 
+static char *logtag()
+{
+	static char buf[20];
+	static int logcnt = 0;
+	static char id = 0;
+
+	/* The `id' is pseudo-random character inteded to lessen
+	   the propablility of reused PID matching same prefix-
+	   string between two SMTP sessions, and thus making the
+	   resulting output  sort(1)able in flat ascii mode.
+	   Timeorder would not be valid, perhaps, but 
+	   For debugging uses, of course  */
+
+	if (id == 0) {
+	  id = '0' + (time(NULL) % 58);
+	  if (id > '9') id += ('A'-'9'+1);
+	  if (id > 'Z') id += ('a'-'Z'+1);
+	}
+
+	sprintf(buf,"%05d%c%05d",pid,id,logcnt);
+	++logcnt;
+	return buf;
+}
+
 int
 main(argc, argv)
 	int argc;
@@ -463,13 +488,30 @@ main(argc, argv)
 	  if (statusreport) report("#hungry");
 
 	  if (fgets(file, sizeof file, stdin) == NULL) break;
+	  if (strchr(file, '\n') == NULL) break; /* No ending '\n' !  Must
+						    have been partial input! */
 
 	  if (emptyline(file, sizeof file))
 	    break;
 
+	  time(&now);
+
 	  s = strchr(file,'\t');
 	  if (s != NULL) {
 	    *s++ = 0;
+
+	    if (host && strcasecmp(host,s)==0) {
+	      extern time_t retryat_time;
+
+	      if (now < retryat_time) {
+		/* Same host, we don't touch on it for a while.. */
+		sleep(2);
+		if (logfp)
+		  fprintf(logfp,"%s#\t(too soon trying to touch on host with 'retryat' diagnostic -- flushing job queue..host='%s')\n",logtag(),host);
+		continue;
+	      }
+	    }
+
 	    /* If different target host, close the old connection.
 	       In theory we could use same host via MX, but...     */
 	    if (host && strcmp(s,host) != 0) {
@@ -478,7 +520,7 @@ main(argc, argv)
 		fclose(smtpfp[0]); smtpfp[0] = NULL;
 		fclose(smtpfp[1]); smtpfp[1] = NULL;
 		if (logfp)
-		  fprintf(logfp, "%d#\t(closed SMTP channel - new host)\n", pid);
+		  fprintf(logfp, "%s#\t(closed SMTP channel - new host)\n", logtag());
 		smtp_isopen = 0;
 	      }
 	    }
@@ -495,9 +537,10 @@ main(argc, argv)
 	    printf("# (fdcnt=%d, file:%s, host:%s)\n", countfds(), file, host);
 	    fflush(stdout);
 	  }
-		       
 
 #ifdef	BIND
+	  res_init();
+
 	  if (checkmx)
 	    dp = ctlopen(file, channel, host, &getout, rightmx, matchroutermxes);
 	  else
@@ -552,7 +595,7 @@ main(argc, argv)
 	  fclose(smtpfp[0]);
 	  fclose(smtpfp[1]);
 	  if (logfp)
-	    fprintf(logfp, "%d#\t(closed SMTP channel - final close)\n", pid);
+	    fprintf(logfp, "%s#\t(closed SMTP channel - final close)\n", logtag());
 	}
 	smtp_isopen = 0;
 
@@ -591,10 +634,9 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 	      if (logfp != NULL && !loggedid) {
 		loggedid = 1;
 		fprintf(logfp,
-			"%d#\t%s: %s\n",
-			pid, dp->msgfile,
+			"%s#\t%s: %s\n",
+			logtag(), dp->msgfile,
 			dp->logident);
-		fflush(logfp);
 	      }
 	      if (!smtp_isopen) {
 		if ((first_uid = atoi(dp->senders->misc)) < 0 ||
@@ -611,7 +653,7 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 		 fclose(smtpfp[0]); smtpfp[0] = NULL;
 		 fclose(smtpfp[1]); smtpfp[1] = NULL;
 		 if (logfp)
-		 fprintf(logfp, "%d#\t(closed SMTP channel - after delivery failure)\n", pid);
+		 fprintf(logfp, "%s#\t(closed SMTP channel - after delivery failure)\n", logtag());
 		 smtp_isopen = 0;
 		 }
 	       */
@@ -867,7 +909,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel - appendlet() failure)\n", pid);
+	      fprintf(logfp, "%s#\t(closed SMTP channel - appendlet() failure)\n", logtag());
 	    smtp_isopen = 0;
 	  }
 	  return EX_TEMPFAIL;
@@ -888,10 +930,10 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 
 	if (logfp != NULL) {
 	  if (r != EX_OK)
-	    fprintf(logfp, "%d#\t%s\n", pid, remotemsg);
+	    fprintf(logfp, "%s#\t%s\n", logtag(), remotemsg);
 	  else
-	    fprintf(logfp, "%d#\t%d bytes, %d in header\n",
-		    pid, size, hsize);
+	    fprintf(logfp, "%s#\t%d bytes, %d in header\n",
+		    logtag(), size, hsize);
 	  fflush(logfp);
 	}
 	for (rp = startrp; rp != endrp; rp = rp->next)
@@ -1377,7 +1419,7 @@ smtpopen(host, fparr, verboselog)
 	char SMTPbuf[1000];
 
 	if (debug) {
-	  fprintf(logfp, "%d#\tsmtpopen: connecting to %s\n", pid, host);
+	  fprintf(logfp, "%s#\tsmtpopen: connecting to %s\n", logtag(), host);
 	  fflush(logfp);
 	}
 	firstmx = 0;
@@ -1416,7 +1458,7 @@ smtpopen(host, fparr, verboselog)
 	      fclose(fparr[0]); fparr[0] = NULL;
 	      fclose(fparr[1]); fparr[1] = NULL;
 	      if (logfp)
-		fprintf(logfp, "%d#\t(closed SMTP channel - HELO failed??)\n", pid);
+		fprintf(logfp, "%s#\t(closed SMTP channel - HELO failed??)\n", logtag());
 	      smtp_isopen = 0;
 	    }
 	    if (i == EX_TEMPFAIL) {
@@ -1428,14 +1470,14 @@ smtpopen(host, fparr, verboselog)
 		fclose(fparr[0]); fparr[0] = NULL;
 		fclose(fparr[1]); fparr[1] = NULL;
 		if (logfp)
-		  fprintf(logfp, "%d#\t(closed SMTP channel - HELO failed(2))\n", pid);
+		  fprintf(logfp, "%s#\t(closed SMTP channel - HELO failed(2))\n", logtag());
 		smtp_isopen = 0;
 	      }
 	    }
 	  }
 	} while (i == EX_TEMPFAIL && firstmx <= mxcount);
 	if (debug) {
-	  fprintf(logfp, "%d#\tsmtpopen: status = %d\n", pid, i);
+	  fprintf(logfp, "%s#\tsmtpopen: status = %d\n", logtag(), i);
 	  fflush(logfp);
 	}
 	return i;
@@ -1460,7 +1502,7 @@ smtpconn(host, fparr, verboselog)
 #endif	/* BIND */
 
 	if (debug) {
-	  fprintf(logfp, "%d#\tsmtpconn: host = %s\n", pid, host);
+	  fprintf(logfp, "%s#\tsmtpconn: host = %s\n", logtag(), host);
 	  fflush(logfp);
 	}
 	if (verboselog)
@@ -1497,11 +1539,11 @@ smtpconn(host, fparr, verboselog)
 	    sprintf(nbuf,"X-IP-addr; [%s]", buf);
 	    notary_setwtt(nbuf);
 	  }
-	  retval = makeconn(hp, &smtpfd, verboselog, fparr, mxcnt);
+	  retval = makeconn(hp, &smtpfd, verboselog, fparr);
 	} else {
 	  hbuf[0] = '\0';
 	  errno = 0;
-#if 0  /* ---------- rip off this gethostbyname() sequence -------- */
+#if 0  /* ---------- chopped off this gethostbyname() sequence -------- */
 	  if ((hp = gethostbyname(host)) != NULL) {
 	    strcpy(hbuf,hp->h_name); /* Get canonic form of target name */
 	    host = hbuf;	/* Save it, we reuse gethostbyname() */
@@ -1543,7 +1585,7 @@ smtpconn(host, fparr, verboselog)
 	    return EX_TEMPFAIL;	/* was EX_NOHOST */
 #endif /* BIND */
 	  }
-#endif /* ---------- rip off this gethostbyname() sequence -------- */
+#endif /* ---------- chopped off this gethostbyname() sequence -------- */
 
 	  mxh[0].host = NULL;	/* use the hostent we got */
 #ifdef	BIND
@@ -1615,8 +1657,10 @@ smtpconn(host, fparr, verboselog)
 	      sprintf(buf,"dns; %s", host);
 	      notary_setwtt(buf);
 	    }
-	    retval = makeconn(hp, &smtpfd, verboselog, fparr, mxcnt);
+	    retval = makeconn(hp, &smtpfd, verboselog, fparr);
 	  } else {
+	    /* Has valid MX records, they have been suitably randomized
+	       at  getmxrr(), and are now ready for use.  */
 	    retval = EX_UNAVAILABLE;
 	    for (i = 0; mxh[i].host != NULL; ++i) {
 	      if ((hp = gethostbyname((char*)mxh[i].host)) == NULL) {
@@ -1631,7 +1675,7 @@ smtpconn(host, fparr, verboselog)
 		sprintf(buf,"dns; %s", mxh[i].host);
 		notary_setwtt(buf);
 	      }
-	      r = makeconn(hp, &smtpfd, verboselog, fparr, mxcnt);
+	      r = makeconn(hp, &smtpfd, verboselog, fparr);
 	      if (r == EX_OK) {
 		while (mxh[i].host != NULL) {
 		  free(mxh[i].host);
@@ -1653,19 +1697,18 @@ smtpconn(host, fparr, verboselog)
 		strcpy(remotehost, host);
 	if (debug) {
 	  fprintf(logfp,
-		  "%d#\tsmtpconn: retval = %d\n", pid, retval);
+		  "%s#\tsmtpconn: retval = %d\n", logtag(), retval);
 	  fflush(logfp);
 	}
 	return retval;
 }
 
 int
-makeconn(hp, fdp, verboselog, fparr, mxcnt)
+makeconn(hp, fdp, verboselog, fparr)
 	struct hostent *hp;
 	int *fdp;
 	FILE *fparr[];
 	FILE *verboselog;
-	int mxcnt;
 {
 	int retval;
 	int fd2;
@@ -1703,7 +1746,7 @@ makeconn(hp, fdp, verboselog, fparr, mxcnt)
 #endif
 	for (hp_init(hp); *hp_getaddr() != NULL; hp_nextaddr()) {
 	  if (matchmyaddress(*hp_getaddr(),hp->h_length)) {
-	    notaryreport(NULL,NULL,"5.4.6 (trying to talk with myself!)","smtp; 500 (Trying to talk with myself!)");
+	    notaryreport(NULL,FAILED,"5.4.6 (trying to talk with myself!)","smtp; 500 (Trying to talk with myself!)");
 	    sprintf(remotemsg,"Trying to talk with myself!");
 	    break;		/* TEMPFAIL or UNAVAILABLE.. */
 	  }
@@ -1713,7 +1756,7 @@ makeconn(hp, fdp, verboselog, fparr, mxcnt)
 	    if (fparr[0] != NULL) fclose(fparr[0]); fparr[0] = NULL;
 	    if (fparr[1] != NULL) fclose(fparr[1]); fparr[1] = NULL;
 	    if (logfp)
-	      fprintf(logfp,"%d#\t(closed SMTP channel at makeconn(), isopen=%d)\n",pid,smtp_isopen);
+	      fprintf(logfp,"%s#\t(closed SMTP channel at makeconn(), isopen=%d)\n",logtag(),smtp_isopen);
 	    smtp_isopen = 0;
 	  }
 
@@ -1766,14 +1809,15 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	u_short p;
 	int errnosave;
 	char *se;
-	time_t now = time(NULL);
+
+	time(&now);
 
 	strcpy(ipaddress, dottedquad((struct in_addr *)ipaddrcp));
 	if (conndebug)
 	  fprintf(stderr, "Trying %s [%s] ... ", hostname, ipaddress);
 	if (logfp)
-	  fprintf(logfp, "%d#\t(Connecting to `%s' [%s] %24.24s)\n",
-		  pid, hostname, ipaddress, ctime(&now));
+	  fprintf(logfp, "%s#\t(Connecting to `%s' [%s] %24.24s)\n",
+		  logtag(), hostname, ipaddress, ctime(&now));
 	if (statusreport) {
 	  strcpy(remotehost,hostname);
 	  report("connecting to [%s]",ipaddress);
@@ -1790,7 +1834,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg+6);
 	  if (logfp)
-	    fprintf(logfp,"%d#\t(Internal error, socket: %s)\n",pid,se);
+	    fprintf(logfp,"%s#\t(Internal error, socket: %s)\n",logtag(),se);
 	  return EX_TEMPFAIL;
 	}
 	if (wantreserved && getuid() == 0) {
@@ -1807,7 +1851,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	      if (conndebug)
 		fprintf(stderr, "%s\n", remotemsg+6);
 	      if (logfp)
-		fprintf(logfp, "%d#\t(Internal error, bind: %s)\n", pid, s);
+		fprintf(logfp, "%s#\t(Internal error, bind: %s)\n", logtag(), s);
 	      return EX_UNAVAILABLE;
 	    }
 	  }
@@ -1819,7 +1863,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg+6);
 	    if (logfp)
-	      fprintf(logfp,"%d#\t(Internal error, too many busy ports)\n", pid);
+	      fprintf(logfp,"%s#\t(Internal error, too many busy ports)\n", logtag());
 	    return EX_TEMPFAIL;
 	  }
 	}
@@ -1855,7 +1899,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	if (verboselog)
 		fprintf(verboselog,"%s\n",remotemsg);
 	if (logfp)
-	  fprintf(logfp,"%d#\t%s\n", pid, remotemsg+4);
+	  fprintf(logfp,"%s#\t%s\n", logtag(), remotemsg+4);
 	switch (errnosave) {	/* from sendmail... */
 	case EISCONN:
 	case ETIMEDOUT:
@@ -1885,7 +1929,7 @@ sig_pipe(sig)
 int sig;
 {
 	if (logfp != NULL) {
-	  fprintf(logfp, "%d#\t*** Received SIGPIPE!\n", pid);
+	  fprintf(logfp, "%s#\t*** Received SIGPIPE!\n", logtag());
 	  fflush(logfp);
 	  abort();
 	}
@@ -1948,8 +1992,8 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	/* we don't need to turn alarms off again (yay!) */
 	fflush(smtpfp[0]);
 	fflush(smtpfp[1]);
-	infd = fileno(smtpfp[0]);
-	outfd = fileno(smtpfp[1]);
+	infd = FILENO(smtpfp[0]);
+	outfd = FILENO(smtpfp[1]);
 
 	if (strbuf != NULL) {
 	  int len = strlen(strbuf) + 2;
@@ -1963,7 +2007,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  if ((r = write(outfd, buf, len)) == -1) {
 	    if (gotalarm) {
 	      strcpy(remotemsg, "Timeout on cmd write");
-	      notaryreport(NULL,"failure","5.4.2 (timeout on cmd write)",
+	      notaryreport(NULL,FAILED,"5.4.2 (timeout on cmd write)",
 			   "smtp; 500 (timeout on cmd write)");
 	    } else
 	      {
@@ -1977,7 +2021,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    smtp_isopen = 0;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel - timeout on smtpwrite())\n", pid);
+	      fprintf(logfp, "%s#\t(closed SMTP channel - timeout on smtpwrite())\n", logtag());
 	    return EX_TEMPFAIL;
 	  } else if (r != len) {
 	    sprintf(remotemsg, "smtp; 500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, len);
@@ -1988,12 +2032,12 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    smtp_isopen = 0;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel - second timeout on smtpwrite() )\n", pid);
+	      fprintf(logfp, "%s#\t(closed SMTP channel - second timeout on smtpwrite() )\n", logtag());
 	    return EX_TEMPFAIL;
 	  }
 	  if (logfp != NULL) {
 	    if (dflag) abort();
-	    fprintf(logfp, "%dw\t%s\n", pid, strbuf);
+	    fprintf(logfp, "%sw\t%s\n", logtag(), strbuf);
 	    fflush(logfp);
 	    dflag = 1;
 	  }
@@ -2010,7 +2054,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	}
 
 	if (debug) {
-	  fprintf(logfp, "%d#\tAttempting to read reply\n",pid);
+	  fprintf(logfp, "%s#\tAttempting to read reply\n",logtag());
 	  fflush(logfp);
 	}
 	if (statusreport && strbuf != NULL) {
@@ -2037,7 +2081,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 		  if (debug)
 		    putc('\n',logfp);
 		  fprintf(logfp,
-			  "%dr\t%s\n", pid, buf);
+			  "%sr\t%s\n", logtag(), buf);
 		  fflush(logfp);
 		}
 
@@ -2066,7 +2110,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 		  if (i > 0)
 		    putc('\n',logfp);
 		  fprintf(logfp,
-			  "%d#\t", pid);
+			  "%s#\t", logtag());
 		}
 		if (isprint(*s)) 
 		  putc(*s, logfp);
@@ -2109,7 +2153,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    smtp_isopen = 0;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel - bad responce on smtpwrite() )\n", pid);
+	      fprintf(logfp, "%s#\t(closed SMTP channel - bad responce on smtpwrite() )\n", logtag());
 	    return EX_TEMPFAIL;
 	  } else {
 	    sprintf(remotemsg, "smtp; 500 (Server hung up on us! Cmd: %s)",
@@ -2122,7 +2166,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
 	    smtp_isopen = 0;
 	    if (logfp)
-	      fprintf(logfp, "%d#\t(closed SMTP channel - hangup on smtpwrite() )\n", pid);
+	      fprintf(logfp, "%s#\t(closed SMTP channel - hangup on smtpwrite() )\n", logtag());
 	    return EX_TEMPFAIL;
 	  }
 	  /* Exit if the last thing we read was a LF and we're on the
@@ -2140,7 +2184,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  fclose(smtpfp[1]); smtpfp[1] = NULL;
 	  smtp_isopen = 0;
 	  if (logfp)
-	    fprintf(logfp, "%d#\t(closed SMTP channel - responce overrun on smtpwrite() )\n", pid);
+	    fprintf(logfp, "%s#\t(closed SMTP channel - responce overrun on smtpwrite() )\n", logtag());
 	  return EX_TEMPFAIL;
 	}
 	*--cp = '\0';	/* kill the LF */
@@ -2154,7 +2198,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  fclose(smtpfp[1]); smtpfp[1] = NULL;
 	  smtp_isopen = 0;
 	  if (logfp)
-	    fprintf(logfp, "%d#\t(closed SMTP channel - unexpected responce on smtpwrite() )\n", pid);
+	    fprintf(logfp, "%s#\t(closed SMTP channel - unexpected responce on smtpwrite() )\n", logtag());
 	  return EX_TEMPFAIL;
 	}
 	--cp;
@@ -2170,7 +2214,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	buf[i] = '\0';
 	response = atoi(buf);
 	if (logfp != NULL) {
-	  fprintf(logfp, "%dr\t%s%c%s\n", pid, buf, ch, &buf[i+1]);
+	  fprintf(logfp, "%sr\t%s%c%s\n", logtag(), buf, ch, &buf[i+1]);
 	  fflush(logfp);
 	}
 	buf[i] = ch;
@@ -2477,7 +2521,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
-	nmx = 0;
+	nmx = *mxcntp;
 	cp = (msgdata *)&answer + sizeof(HEADER);
 	for (; qdcount > 0; --qdcount) {
 #if	defined(BIND_VER) && (BIND_VER >= 473)
@@ -2521,7 +2565,6 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 					       should not happen in case
 					       of MX server, though..    */
 	  if (!hent) {
-	    *mxcntp += 1;
 	    continue;		/* Well well.. spurious! */
 	  }
 	  if (cistrcmp(hent->h_name, myhostname) == 0
@@ -2536,6 +2579,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	  strcpy((char*)mx[nmx].host, (char*)buf);
 	  /* [mea] Canonicalize this target & get A records.. */
 	  ++nmx;
+	  *mxcntp = nmx;
 	}
 	if (nmx == 0 && realname[0] != '\0' &&
 	    cistrcmp(host,(char*)realname)) {
@@ -2547,8 +2591,6 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	  mxcount = 0;
 	  return EX_OK;
 	}
-	for (i = 0; i < nmx; ++i) {
-	}
 
 	/* discard MX RRs with a value >= that of  myhost */
 	if (maxpref >= 0) {
@@ -2567,22 +2609,22 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	}
 #ifdef	RFC974
 	/* discard MX's that do not support SMTP service */
-	for (n = 0; checkwks && n < nmx; ++n) {
-	  if (mx[n].host == NULL)
-	    continue;
-	  strcpy((char*)buf, (char*)mx[n].host);
-	  /* It is an MX, it CAN'T have CNAME ! */
-	  if (!getrrtype((void*)buf, sizeof buf, T_WKS, 0)) {
-	    free(mx[n].host);
-	    mx[n].host = NULL;
+	if (checkwks)
+	  for (n = 0; n < nmx; ++n) {
+	    if (mx[n].host == NULL)
+	      continue;
+	    strcpy((char*)buf, (char*)mx[n].host);
+	    /* It is an MX, it CAN'T have CNAME ! */
+	    if (!getrrtype((void*)buf, sizeof buf, T_WKS, 0)) {
+	      free(mx[n].host);
+	      mx[n].host = NULL;
+	    }
 	  }
-	}
 #endif	/* RFC974 */
 	/* determine how many are left */
 	for (i = 0, n = 0; i < nmx; ++i) {
 	  if (mx[i].host == NULL)
 	    continue;
-	  *mxcntp += 1;
 	  if (n < i) {
 	    mx[n] = mx[i];
 	    mx[i].host = NULL;
@@ -2595,6 +2637,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	  return EX_UNAVAILABLE;
 	}
 	nmx = n;
+	*mxcntp = nmx;
 	/* sort the records */
 	for (i = 0; i < nmx; i++) {
 	  for (j = i + 1; j < nmx; j++) {
